@@ -1,4 +1,4 @@
-/* Copyright (C) 2009, 2010 D. V. Wiebe
+/* Copyright (C) 2009-2015 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -23,18 +23,18 @@
 static PyObject *GdPy_DirfileError;
 static const char *gdpy_exception_list[GD_N_ERROR_CODES] = {
   NULL,
-  "Open",
+  NULL, /* 1 */
   "Format",
-  "Truncate",
+  NULL, /* 3 */
   "Creation",
   "BadCode",
   "BadType",
-  "RawIO",
-  "OpenFragment",
-  "InternalError",
+  "IO",
+  NULL, /* 8 */
+  "Internal",
   "Alloc",
   "Range",
-  "OpenLinfile",
+  "LUT",
   "RecurseLevel",
   "BadDirfile",
   "BadFieldType",
@@ -49,23 +49,80 @@ static const char *gdpy_exception_list[GD_N_ERROR_CODES] = {
   "BadReference",
   "Protected",
   "Deletion",
-  "BadEndianess",
+  "Argument",
   "Callback",
-  "BadProtection",
+  "Exists",
   "UncleanDatabase",
   "Domain",
   "BadRepr",
-  "BadVersion",
-  "Flush"
+  NULL, /* 33 */
+  NULL, /* 34 */
+  "Bounds",
+  "LineTooLong"
 };
 PyObject *gdpy_exceptions[GD_N_ERROR_CODES];
 
-int gdpy_convert_from_pyobj(PyObject* value, union gdpy_quadruple_value *data,
+/* These are unused but for backwards compatibility are defined as aliases of
+ * current exceptions */
+static struct {
+  const char *name;
+  int e;
+} gdpy_dead_exceptions[] = {
+  { "BadEndianness", GD_E_ARGUMENT },
+  { "BadProtection", GD_E_ARGUMENT },
+  { "BadVersion", GD_E_ARGUMENT },
+  { "OpenLinfile", GD_E_LUT },
+  { "Flush", GD_E_IO },
+  { "Open", GD_E_IO },
+  { "OpenFragment", GD_E_IO },
+  { "OpenFragment", GD_E_IO },
+  { "OpenInclude", GD_E_IO },
+  { "RawIO", GD_E_IO },
+  { "Trunc", GD_E_IO },
+  { NULL, 0}
+};
+
+/* Like PyList_Append, but steal the object's reference */
+int gdpylist_append(PyObject *list, PyObject *item)
+{
+  dtrace("%p, %p", list, item);
+  int ret;
+
+  ret = PyList_Append(list, item);
+  Py_DECREF(item);
+
+  dreturn("%i", ret);
+  return ret;
+}
+
+/* Create an array of strings from a NULL-terminated string list */
+PyObject *gdpy_to_pystringlist(const char **list)
+{
+  PyObject *pyobj;
+  size_t i;
+
+  dtrace("%p", list);
+
+  pyobj = PyList_New(0);
+  if (pyobj)
+    for (i = 0; list[i] != NULL; ++i)
+      if (gdpylist_append(pyobj, PyString_FromString(list[i]))) {
+        Py_DECREF(pyobj);
+        pyobj = NULL;
+        break;
+      }
+
+  dreturn("%p", pyobj);
+  return pyobj;
+}
+
+int gdpy_convert_from_pyobj(PyObject *value, union gdpy_quadruple_value *data,
     gd_type_t type)
 {
-  dtrace("%p, %p, %02x", value, data, type);
   int data_type;
 
+  dtrace("%p, %p, %02x", value, data, type);
+
   /* check value type, and figure out autotype, if needed */
   if (PyInt_Check(value)) {
     data->s = PyInt_AsLong(value);
@@ -147,7 +204,7 @@ int gdpy_convert_from_pyobj(PyObject* value, union gdpy_quadruple_value *data,
       return -1;
     }
   } else if (PyComplex_Check(value)) {
-    data->c = gdpy_as_complex(value);
+    gdpy_as_complex(gd_csp_(data->c), value);
     data_type = GDPY_COMPLEX_AS_COMPLEX;
 
     if (PyErr_Occurred()) {
@@ -164,16 +221,17 @@ int gdpy_convert_from_pyobj(PyObject* value, union gdpy_quadruple_value *data,
   return data_type;
 }
 
-gd_type_t gdpy_convert_from_pylist(PyObject* value, void *data, gd_type_t type,
+gd_type_t gdpy_convert_from_pylist(PyObject *value, void *data, gd_type_t type,
     size_t ns)
 {
   size_t i;
+  int data_type;
   union gdpy_quadruple_value tmp;
 
   dtrace("%p, %p, %02x, %zi", value, data, type, ns);
 
   /* use the first element to determine the data type */
-  int data_type = gdpy_convert_from_pyobj(PyList_GetItem(value, 0), &tmp, type);
+  data_type = gdpy_convert_from_pyobj(PyList_GetItem(value, 0), &tmp, type);
 
   if (data_type == -1) {
     dreturn("%02x", GD_UNKNOWN);
@@ -214,9 +272,9 @@ gd_type_t gdpy_convert_from_pylist(PyObject* value, void *data, gd_type_t type,
       break;
     case GDPY_COMPLEX_AS_COMPLEX:
       type = GD_COMPLEX128;
-      *(complex double*)data = tmp.c;
+      gd_cs2ca_(data, 0, tmp.c, double);
       for (i = 1; i < ns; ++i)
-        ((double complex*)data)[i] = gdpy_as_complex(PyList_GetItem(value, i));
+        gdpy_as_complex(((double*)data) + 2 * i, PyList_GetItem(value, i));
       break;
   }
 
@@ -225,7 +283,6 @@ gd_type_t gdpy_convert_from_pylist(PyObject* value, void *data, gd_type_t type,
 }
 
 /* generic utitily functions */
-#ifdef USE_NUMPY
 gd_type_t gdpy_type_from_npytype(int npytype)
 {
   gd_type_t type;
@@ -349,102 +406,100 @@ int gdpy_npytype_from_type(gd_type_t type)
   dreturn("%i", npytype);
   return npytype;
 }
-#endif
 
-PyObject* gdpy_convert_to_pylist(const void* data, gd_type_t type, size_t ns)
+PyObject *gdpy_convert_to_pylist(const void *data, gd_type_t type, size_t ns)
 {
   size_t i;
+  PyObject *pyobj;
 
   dtrace("%p, %02x, %zi", data, type, ns);
 
-  PyObject* pylist;
-
-  if (type != GD_NULL)
-    pylist = PyList_New(0);
+  if (type == GD_NULL) {
+    Py_INCREF(Py_None);
+    dreturn("%p", Py_None);
+    return Py_None;
+  }
+  pyobj = PyList_New(0);
 
   switch(type) {
-    case GD_NULL:
-      Py_INCREF(Py_None);
-      dreturn("%p", Py_None);
-      return Py_None;
     case GD_UINT8:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((uint8_t*)data)[i])))
+        if (gdpylist_append(pyobj, PyInt_FromLong((long)((uint8_t*)data)[i])))
           return NULL;
       break;
     case GD_INT8:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((int8_t*)data)[i])))
+        if (gdpylist_append(pyobj, PyInt_FromLong((long)((int8_t*)data)[i])))
           return NULL;
       break;
     case GD_UINT16:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((uint16_t*)data)[i])))
+        if (gdpylist_append(pyobj, PyInt_FromLong((long)((uint16_t*)data)[i])))
           return NULL;
       break;
     case GD_INT16:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((int16_t*)data)[i])))
+        if (gdpylist_append(pyobj, PyInt_FromLong((long)((int16_t*)data)[i])))
           return NULL;
       break;
     case GD_UINT32:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
+        if (gdpylist_append(pyobj,
               PyLong_FromUnsignedLong((unsigned long)((uint32_t*)data)[i])))
           return NULL;
       break;
     case GD_INT32:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((int32_t*)data)[i])))
+        if (gdpylist_append(pyobj, PyInt_FromLong((long)((int32_t*)data)[i])))
           return NULL;
       break;
     case GD_UINT64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyLong_FromUnsignedLongLong(
+        if (gdpylist_append(pyobj, PyLong_FromUnsignedLongLong(
                 (unsigned long long)((uint64_t*)data)[i])))
           return NULL;
       break;
     case GD_INT64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
+        if (gdpylist_append(pyobj,
               PyLong_FromLongLong((long long)((int64_t*)data)[i])))
           return NULL;
       break;
     case GD_FLOAT32:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
+        if (gdpylist_append(pyobj,
               PyFloat_FromDouble((double)((float*)data)[i])))
           return NULL;
       break;
     case GD_FLOAT64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyFloat_FromDouble(((double*)data)[i])))
+        if (gdpylist_append(pyobj, PyFloat_FromDouble(((double*)data)[i])))
           return NULL;
       break;
     case GD_COMPLEX64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
-              gdpy_from_complex((double complex)((float complex*)data)[i])))
+        if (gdpylist_append(pyobj, gdpy_from_complexp(((float*)data) + 2 * i)))
           return NULL;
       break;
     case GD_COMPLEX128:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
-              gdpy_from_complex(((double complex*)data)[i])))
+        if (gdpylist_append(pyobj, gdpy_from_complexp(((double*)data) + 2 * i)))
           return NULL;
       break;
+    case GD_NULL:
     case GD_UNKNOWN: /* prevent compiler warning */
       break;
   }
 
-  dreturn("%p", pylist);
-  return pylist;
+  dreturn("%p", pyobj);
+  return pyobj;
 }
 
-PyObject* gdpy_convert_to_pyobj(const void* data, gd_type_t type)
+PyObject *gdpy_convert_to_pyobj(const void *data, gd_type_t type)
 {
+  PyObject *pyobj = NULL;
+
   dtrace("%p, %02x", data, type);
-  PyObject* pyobj = NULL;
 
   switch(type) {
     case GD_NULL:
@@ -482,10 +537,10 @@ PyObject* gdpy_convert_to_pyobj(const void* data, gd_type_t type)
       pyobj = PyFloat_FromDouble(*(double*)data);
       break;
     case GD_COMPLEX64:
-      pyobj = gdpy_from_complex((double complex)*(float complex*)data);
+      pyobj = gdpy_from_complexp((float*)data);
       break;
     case GD_COMPLEX128:
-      pyobj = gdpy_from_complex(*(double complex*)data);
+      pyobj = gdpy_from_complexp((double*)data);
       break;
     case GD_UNKNOWN: /* prevent compiler warning */
       break;
@@ -495,15 +550,58 @@ PyObject* gdpy_convert_to_pyobj(const void* data, gd_type_t type)
   return pyobj;
 }
 
+static PyObject *gdpy_encoding_support(struct gdpy_fragment_t *self,
+    PyObject *args, PyObject *keys)
+{
+  char *keywords[] = { "encoding", NULL };
+  unsigned long enc;
+  PyObject *pyobj;
+  int n;
+
+  dtrace("%p, %p, %p", self, args, keys);
+
+  if (!PyArg_ParseTupleAndKeywords(args, keys, "k:pygetdata.encoding_support",
+        keywords, &enc))
+  {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  n = gd_encoding_support(enc);
+
+  if (n == 0) {
+    Py_INCREF(Py_None);
+    dreturn("%p", Py_None);
+    return Py_None;
+  }
+
+  pyobj = PyInt_FromLong(n);
+
+  dreturn("%p", pyobj);
+  return pyobj;
+}
+
 /* GetData */
 static PyMethodDef GetDataMethods[] = {
+  { "encoding_support", (PyCFunction)gdpy_encoding_support,
+    METH_VARARGS | METH_KEYWORDS, "encoding_support(encoding)\n\n"
+      "The 'encoding' parameter should be one of the pygetdata.*_ENCODED\n"
+      "symbols.  This method will return pygetdata.RDWR if the library can\n"
+      "read and write the encoding, pygetdata.RDONLY if the library can\n"
+      /* ------- handy ruler ---------------------------------------------| */
+      "only read the encodin, or None otherwise.  See\n"
+      "gd_encoding_support(3)."
+  },
   { NULL, NULL, 0, NULL }
 };
 
 PyMODINIT_FUNC initpygetdata(void)
 {
   int i;
-  PyObject* mod;
+  PyObject *mod;
+  PyObject *mdict;
+
+  dtracevoid();
 
   if (PyType_Ready(&gdpy_dirfile) < 0)
     return;
@@ -514,11 +612,9 @@ PyMODINIT_FUNC initpygetdata(void)
   if (PyType_Ready(&gdpy_fragment) < 0)
     return;
 
-#ifdef USE_NUMPY
   /* The following macro will cause this function to return if importing numpy
    * fails */
   import_array()
-#endif
 
   mod = Py_InitModule3("pygetdata", GetDataMethods,
       "Bindings to the GetData library for Dirfile access\n\n"
@@ -586,20 +682,14 @@ PyMODINIT_FUNC initpygetdata(void)
 
   /* author */
   PyModule_AddStringConstant(mod, "__author__",
-      "D. V. Wiebe <getdata@ketiltrout.net>");
+      "The GetData Project <http://getdata.sourceforge.net/>");
 
   /* add constants */
   for (i = 0; gdpy_constant_list[i].name != NULL; ++i)
     PyModule_AddIntConstant(mod, gdpy_constant_list[i].name,
         gdpy_constant_list[i].value);
 
-  PyModule_AddIntConstant(mod, "__numpy_supported__",
-#ifdef USE_NUMPY
-      1
-#else
-      0
-#endif
-      );
+  PyModule_AddIntConstant(mod, "__numpy_supported__", 1);
 
   /* add exceptions */
   GdPy_DirfileError = PyErr_NewException("pygetdata.DirfileError",
@@ -608,10 +698,26 @@ PyMODINIT_FUNC initpygetdata(void)
   PyModule_AddObject(mod, "DirfileError", GdPy_DirfileError);
 
   for (i = 1; i < GD_N_ERROR_CODES; ++i) {
-    char name[40];
-    sprintf(name, "pygetdata.%sError", gdpy_exception_list[i]);
-    gdpy_exceptions[i] = PyErr_NewException(name, GdPy_DirfileError, NULL);
-    Py_INCREF(gdpy_exceptions[i]);
-    PyModule_AddObject(mod, name + 10, gdpy_exceptions[i]);
+    if (gdpy_exception_list[i]) {
+      char name[40];
+      sprintf(name, "pygetdata.%sError", gdpy_exception_list[i]);
+      gdpy_exceptions[i] = PyErr_NewException(name, GdPy_DirfileError, NULL);
+      Py_INCREF(gdpy_exceptions[i]);
+      PyModule_AddObject(mod, name + 10, gdpy_exceptions[i]);
+    } else
+      gdpy_exceptions[i] = GdPy_DirfileError;
   }
+
+  /* add dead exceptions -- we do this through manual dictionary editing */
+  mdict = PyModule_GetDict(mod);
+  if (mdict)
+    for (i = 0; gdpy_dead_exceptions[i].name; ++i) {
+      char name[40];
+      sprintf(name, "%sError", gdpy_dead_exceptions[i].name);
+      Py_INCREF(gdpy_exceptions[gdpy_dead_exceptions[i].e]);
+      PyDict_SetItemString(mdict, name,
+          gdpy_exceptions[gdpy_dead_exceptions[i].e]);
+    }
+
+  dreturnvoid();
 }
