@@ -2,9 +2,101 @@ C++ BINDINGS FOR GETDATA
 ========================
 
 This README describes the C++ bindings for the GetData library.  These bindings
-consist of a several C++ classes in the `GetData' namespace.  Header files
-defining these classes are installed into the ${includedir}/getdata directory.
-The following classes are available:
+consist of a several C++ classes in the `GetData' namespace, plus a few
+associated named constants.  Header files defining these data types are
+installed into the ${includedir}/getdata directory.  The classes defined in
+the GetData namespace are: Dirfile, Entry, Fragment, plus an additional Entry
+subclass for each field type (i.e. RawEntry, LincomEntry, &c.)
+
+CONSTANTS
+=========
+
+Because the C++ headers include the C API header getdata.h, all named constants
+defined in the C API may also be used in the C++ API without change in meaning.
+
+In addition to the classes documented below, the C++ bindings define the
+following enumerated constants.  These are re-encapsulations of various C API
+named constants permitting better type safety in C++.  In all cases, the
+corresponding C API constant can be used in place of the C++ named constant,
+with an appropriate type-cast (so that, e.g., (GetData::DataType)GD_NULL can
+be used in place of GetData::Null).
+
+The GetData data types (defined in getdata/entry.h):
+
+  enum DataType {
+    Null      = GD_NULL,      Unknown    = GD_UNKNOWN,
+    UInt8     = GD_UINT8,     Int8       = GD_INT8,
+    UInt16    = GD_UINT16,    Int16      = GD_INT16,
+    UInt32    = GD_UINT32,    Int32      = GD_INT32,
+    UInt64    = GD_UINT64,    Int64      = GD_INT64,
+    Float32   = GD_FLOAT32,   Float64    = GD_FLOAT64,
+    Complex64 = GD_COMPLEX64, Complex128 = GD_COMPLEX128
+  };
+
+The GetData entry types (defined in getdata/entry.h):
+
+  enum EntryType {
+    NoEntryType       = GD_NO_ENTRY,
+    RawEntryType      = GD_RAW_ENTRY,
+    LincomEntryType   = GD_LINCOM_ENTRY,
+    LinterpEntryType  = GD_LINTERP_ENTRY,
+    BitEntryType      = GD_BIT_ENTRY,
+    MultiplyEntryType = GD_MULTIPLY_ENTRY,
+    PhaseEntryType    = GD_PHASE_ENTRY,
+    SBitEntryType     = GD_SBIT_ENTRY,
+    PolynomEntryType  = GD_POLYNOM_ENTRY,
+    ConstEntryType    = GD_CONST_ENTRY,
+    CarrayEntryType   = GD_CARRAY_ENTRY,
+    StringEntryType   = GD_STRING_ENTRY,
+    IndexEntryType    = GD_INDEX_ENTRY,
+    DivideEntryType   = GD_DIVIDE_ENTRY,
+    RecipEntryType    = GD_RECIP_ENTRY,
+    WindowEntryType   = GD_WINDOW_ENTRY,
+    MPlexEntryType    = GD_MPLEX_ENTRY
+  };
+
+The GetData window operations (defined in getdata/entry.h):
+
+  enum WindOpType {
+    WindOpEq = GD_WINDOP_EQ,
+    WindOpNe = GD_WINDOP_NE,
+    WindOpGe = GD_WINDOP_GE,
+    WindOpGt = GD_WINDOP_GT,
+    WindOpLe = GD_WINDOP_LE,
+    WindOpLt = GD_WINDOP_LT,
+    WindOpSet = GD_WINDOP_SET,
+    WindOpClr = GD_WINDOP_CLR
+  };
+
+The GetData encoding schemes (defined in getdata/fragment.h):
+
+  enum EncodingScheme {
+    AutoEncoding   = GD_AUTO_ENCODED, 
+    Bzip2Encoding  = GD_BZIP2_ENCODED,
+    FlacEncoding   = GD_FLAC_ENCODED,
+    GzipEncoding   = GD_GZIP_ENCODED,
+    RawEncoding    = GD_UNENCODED,
+    SieEncoding    = GD_SIE_ENCODED,  
+    SlimEncoding   = GD_SLIM_ENCODED,
+    TextEncoding   = GD_TEXT_ENCODED,
+    ZzipEncoding   = GD_ZZIP_ENCODED,
+    ZzslimEncoding = GD_ZZSLIM_ENCODED, 
+    UnsupportedEncoding = GD_ENC_UNSUPPORTED
+  };
+
+NON-MEMBER FUNCTIONS
+====================
+
+The following non-member functions in the GetData namespace are defined in
+getdata/dirfile.h:
+
+* GetData::EncodingSupport(GetData::EncodingScheme encoding)
+
+  This function returns GD_RDWR if the library supports both reading from and
+  writing to the specified encoding, GD_RDONLY, if the library can only read
+  from the encoding, or -1 if the library supports neither reading nor writing
+  for the specified encoding.  See gd_encoding_support(3).
+
 
 DIRFILE CLASS
 =============
@@ -47,8 +139,14 @@ are available:
 
 * int Dirfile::Error()
 
-  The Error method provides access to the error member of the underlying
-  DIRFILE* object.
+  The Error method calls gd_error(3) to return the error status of the last
+  GetData library call on this Dirfile object.
+
+* int Dirfile::ErrorCount()
+
+  The ErrorCount method calls gd_error_count(3) to return the number of errors
+  encountered by the GetData library on this Dirfile object since this method
+  was last called (or, the first time, since the object was created).
 
 * const char *Dirfile::ErrorString()
 * const char *Dirfile::ErrorString(size_t len)
@@ -127,14 +225,21 @@ are available:
   retained.
 
 * int Dirfile::Add(const Entry &entry)
-* int Dirfile::AddSpec(const char *spec, int format_file = 0)
+* int Dirfile::AddAlias(const char* field_code, const char* target,
+    int fragment_index = 0)
+* int Dirfile::AddSpec(const char *spec, int fragment_index = 0)
+* const char** Dirfile::Aliases(const char* field_code)
+* const char* Dirfile::AliasTarget(const char* field_code)
 * int Dirfile::AlterSpec(const char *line, int recode = 0)
+* const size_t Dirfile::ArrayLen(const char *field_code)
 * const gd_carray_t *Dirfile::Carrays(GetData::DataType type = Float64)
-* const size_t Dirfile::CarrayLen(const char *field_code)
 * const void *Dirfile::Constants(GetData::DataType type = Float64)
-* int Delete(const char *field_code, int flags = 0)
+* int Dirfile::Delete(const char *field_code, unsigned flags = 0)
+* const char **Dirfile::EntryList(const char *parent = NULL, int type = 0,
+    unsigned int flags = 0)
 * const char **Dirfile::FieldList()
 * const char **Dirfile::FieldListByType(GetData::EntryType type)
+* unsigned long Dirfile::Flags(unsigned long set = 0, unsigned long reset = 0)
 * int Dirfile::Flush(const char *field_code = NULL)
 * const char *Dirfile::FormatFilename(int index)
 * int Dirfile::GetCarray(const char *field_code, GetData::DataType type,
@@ -145,19 +250,30 @@ are available:
     off_t first_sample, size_t num_frames, size_t num_samples,
     GetData::DataType type, void *data_out)
 * size_t Dirfile::GetString(const char *field_code, size_t len, char *data_out)
-* int Include(const char *file, int format_file, unsigned int flags)
+* int Dirfile::Hidden(const char* field_code)
+* int Dirfile::Hide(const char* field_code)
+* int Dirfile::Include(const char *file, int fragment_index, unsigned int flags)
+* int Dirfile::IncludeAffix(const char *file, int fragment_index = 0,
+    const char* prefix = NULL, const char* suffix = NULL,
+    unsigned long flags = 0)
 * int Dirfile::MAdd(const Entry &entry, const char *parent)
+* int Dirfile::MAddAlias(const char* parent, const char* name,
+    const char* target)
 * int Dirfile::MAddSpec(const char *spec, const char *parent)
 * int Dirfile::MAlterSpec(const char *line, const char *parent)
 * const gd_carray_t *Dirfile::MCarrays(GetData::DataType type = Float64)
 * const void *Dirfile::MConstants(const char *parent, GetData::DataType type)
+* int Dirfile::MetaFlush()
 * const char **Dirfile::MFieldList(const char *parent)
 * const char **Dirfile::MFieldListByType(const char *parent,
-    * GetData::EntryType type)
-* int Dirfile::MetaFlush()
+    GetData::EntryType type)
+* void MplexLookback(int lookback)
 * const char **Dirfile::MStrings(const char *parent)
 * const char **Dirfile::MVectorList(const char *parent)
+* int Dirfile::NAliases(const char* field_code)
 * DataType Dirfile::NativeType(const char *field_code)
+* unsigned int Dirfile::NEntries(const char *parent = NULL, int type = 0,
+    unsigned int flags = 0)
 * unsigned int Dirfile::NFields()
 * unsigned int Dirfile::NFieldsByType(GetData::EntryType type)
 * off_t Dirfile::NFrames()
@@ -175,29 +291,25 @@ are available:
     off_t first_sample, size_t num_frames, size_t num_samples,
     GetData::DataType type, const void *data_in)
 * size_t Dirfile::PutString(const char *field_code, const char *data_in)
+* int RawClose(const char *field_code = NULL)
 * unsigned int Dirfile::SamplesPerFrame(const char *field_code)
+* off_t Dirfile::Seek(const char *field_code, off_t frame_num, off_t sample_num,
+    int whence)
 * const char **Dirfile::Strings()
+* char *StrTok(const char *string = NULL)
+* int Dirfile::Sync(const char* field_code = NULL)
+* off_t Dirfile::Tell(const char *field_code)
+* int Dirfile::UnHide(const char* field_code)
 * int Dirfile::Validate(const char *field_code)
 * const char **Dirfile::VectorList()
+* int VerbosePrefix(const char *prefix = NULL)
 
   These methods call the corresponding function from the C API on the C DIRFILE
-  object associated with the C++ object.  Arguments of type GetData::DataType
-  should be one of:
+  object associated with the C++ object.  For allowed values for arguments of
+  type GetData::DataType or GetData::EntryType, see the CONSTANTS section above.
 
-    Null, Unknown, UInt8, Int8, UInt16, Int16, UInt32, Int32, UInt64, Int64,
-    Float32, Float64, Complex64, Complex128
-
-  which are aliases for the gd_type_t values GD_NULL, GD_UNKNOWN, GD_UINT8, &c.
-  Arguments of type GetData::EntryType should be one of 
-
-    NoEntryType, BitEntryType, CarrayEntry, ConstEntryType, DivideEntryType,
-    IndexEntryType, LincomEntryType, LinterpEntryType, MultiplyEntryType,
-    PhaseEntryType, PolynomEntryType, RawEntryType, RecipEntryType,
-    SBitEntryType, StringEntryType
-
-  which are aliases for the gd_entype_t values GD_NO_ENTRY, GD_RAW_ENTRY, &c.
-  Note that the arguments to AddSpec are opposite of the corresponding function
-  in add_spec.
+  Note that the arguments to AddSpec are opposite to that of the corresponding
+  function add_spec(3) in the C API.
 
 
 FRAGMENT CLASS
@@ -220,21 +332,19 @@ be incorrect, and all pre-existing Fragment objects should be destroyed.
 * int Fragment::Index()
 
   These methods return the specified information on the associated fragment.
-  Variables of type EncodingScheme will be one of
-  
-    AutoEncoding, RawEncoding, TextEncoding, SlimEncoding, GzipEncoding,
-    Bzip2Encoding, UnsupportedEncoding
-
-  which are aliases for GD_AUTO_ENCODED, GD_UNENCODED, GD_TEXT_ENCODED, &c.
+  For allowed values for arguments of type EncodingScheme, see the CONSTANTS
+  section above.
 
 * int SetEncoding(EncodingScheme encoding, int recode = 0)
 * int SetEndianness(unsigned long byte_sex, int recode = 0)
 * int SetFrameOffset(off_t offset, int recode = 0)
+* int SetPrefix(const char *prefix)
 * int SetProtection(int protection_level)
+* int SetSuffix(const char *suffix)
 
   These methods set the specified information on the associated fragment by
   calling gd_alter_encoding(3), gd_alter_endianness(3), gd_alter_frameoffset(3),
-  or gd_protect(3) as appropriate.
+  gd_protect(3), or gd_alter_affixes(3) as appropriate.
 
 
 ENTRY CLASS
@@ -267,34 +377,31 @@ The following methods are available:
 
 * EntryType Entry::Type()
 
-  This will return the field type of the Entry's field.  This will be one of:
-
-    NoEntryType, BitEntryType, ConstEntryType, DivideEntryType, IndexEntryType,
-    LincomEntryType, LinterpEntryType, MultiplyEntryType, PhaseEntryType,
-    PolynomEntryType, RawEntryType, RecipEntryType, SBitEntryType,
-    StringEntryType
+  This will return the field type of the Entry's field.  This will be one of the
+  Entry types listed above in the CONSTANTS section.
 
 * int Entry::SetFragmentIndex(int fragment_index)
-* int Entry::Move(int new_fragment, int move_data = 0)
+* int Entry::Move(int new_fragment, unsigned flags = 0)
 
   These will update the fragment index of the entry.  If the entry is
   associated, these will call gd_move(3) to move the field to a different
   fragment.  These two functions are equivalent, except Entry::Move allows
-  specifying the move_data flag.  Entry::SetFragmentIndex always calls gd_move
-  with move_data = 0.
+  specifying flags for the move.  Entry::SetFragmentIndex always calls gd_move
+  with flags = 0.
 
 * const char *Entry::Name()
 
   This method returns the name of the field.  
 
 * int Entry::SetName(const char *new_name)
-* int Entry::Rename(const char *new_name, int move_data = 0)
+* int Entry::Rename(const char *new_name, unsigned flags = 0)
 
   These will change the name of the field of this entry.  If the entry object
   is associated, these will also call calling gd_rename(3).  These two functions
-  are equivalent, except Entry::Rename allows specifying the move_data flag
-  explicitly.  Entry::SetName always calls gd_rename with move_data = 0.
+  are equivalent, except Entry::Rename allows specifying the flags explicitly.
+  Entry::SetName always calls gd_rename with flags = 0.
 
+* unsigned int Entry::Flags()
 * virtual int Entry::ComplexScalars()
 * virtual int Entry::FragmentIndex()
 * virtual int Entry::PolyOrd()
@@ -307,25 +414,29 @@ The following methods are available:
 * virtual DataType Entry::ConstType()
 * virtual size_t Entry::ArrayLen()
 * virtual const char *Entry::Table()
+* virtual WindOpType Entry::WindOp()
+* virtual gd_triplet_t Entry::Threshold()
+* virtual int Entry::CountVal()
+* virtual int Entry::Period()
 
   These methods will return the corresponding member of the gd_entry_t object.
   Only methods reasonable to be queried for the given field type will return
   meaningful results.
 
-* virtual const char *Entry::Input(int index = 0)
-* virtual double Entry::Scale(int index = 0)
-* virtual std::complex<double> Entry::CScale(int index = 0)
-* virtual double Entry::Offset(int index = 0)
-* virtual std::complex<double> Entry::COffset(int index = 0)
-* virtual double Entry::Coefficient(int index = 0)
-* virtual std::complex<double> Entry::CCoefficient(int index = 0)
-* virtual const char *Entry::Scalar(int index = 0)
-* virtual int Entry::ScalaIndexr(int index = 0)
+* virtual const char *Entry::Input(int index)
+* virtual double Entry::Scale(int index)
+* virtual std::complex<double> Entry::CScale(int index)
+* virtual double Entry::Offset(int index)
+* virtual std::complex<double> Entry::COffset(int index)
+* virtual double Entry::Coefficient(int index)
+* virtual std::complex<double> Entry::CCoefficient(int index)
+* virtual const char *Entry::Scalar(int index)
+* virtual int Entry::ScalaIndex(int index)
 
   These methods will return an element from the gd_entry_t members in_fields[],
   m[], or b[], indexed by the supplied parameter.  Attempts to access elements
-  out of range for the field that the Entry class describes will not return
-  meaningful results.
+  out of range for the field that the Entry class describes will return zero
+  or NULL.
 
 
 ENTRY CHILD CLASSES
@@ -347,7 +458,7 @@ Defined in getdata/rawentry.h
   This creates a new RAW entry object with default parameters.
 
 * RawEntry::RawEntry(const char *field_code, DataType data_type,
-    unsigned int spf, int format_file = 0)
+    unsigned int spf, int fragment_index = 0)
 
 * const char *RawEntry::FileName()
 
@@ -356,7 +467,8 @@ Defined in getdata/rawentry.h
 
 * virtual unsigned int RawEntry::SamplesPerFrame()
 * virtual DataType RawEntry::RawType()
-* virtual const char *Scalar()
+* virtual const char *Scalar(int index = 0)
+* virtual int ScalarIndex(int index = 0)
 
   These methods, re-implemented from the Entry class, return the corresponding
   field parameter.
@@ -381,31 +493,31 @@ Defined in getdata/lincomentry.h
   This creates a new LINCOM entry object with default parameters.
 
 * LincomEntry::LincomEntry(const char *field_code, int n_fields,
-    const char **in_fields, double *m, double *b, int format_file = 0)
+    const char **in_fields, double *m, double *b, int fragment_index = 0)
 * LincomEntry::LincomEntry(const char *field_code, int n_fields,
     const char **in_fields, std::complex<double> *m, std::complex<double> *b,
-    int format_file = 0)
+    int fragment_index = 0)
 
-* virtual const char *LincomEntry::Input(int index = 0)
+* virtual const char *LincomEntry::Input(int index)
 * virtual int LincomEntry::ComplexScalars()
 * virtual int LincomEntry::NFields()
-* virtual double LincomEntry::Scale(int index = 0)
-* virtual std::complex<double> LincomEntry::CScale(int index = 0)
-* virtual double LincomEntry::Offset(int index = 0)
-* virtual std::complex<double> LincomEntry::COffset(int index = 0)
-* virtual const char *LincomEntry::Scalar(int index = 0)
-* virtual int LincomEntry::ScalarIndex(int index = 0)
+* virtual double LincomEntry::Scale(int index)
+* virtual std::complex<double> LincomEntry::CScale(int index)
+* virtual double LincomEntry::Offset(int index)
+* virtual std::complex<double> LincomEntry::COffset(int index)
+* virtual const char *LincomEntry::Scalar(int index)
+* virtual int LincomEntry::ScalarIndex(int index)
 
   These methods, re-implemented from the Entry class, return the corresponding
   field parameter.
 
-* int LincomEntry::SetInput(const char *field, int index = 0)
-* int LincomEntry::SetScale(double scale, int index = 0)
-* int LincomEntry::SetScale(const char* scale, int index = 0)
-* int LincomEntry::SetScale(std::complex<double> scale, int index = 0)
-* int LincomEntry::SetOffset(double offset, int index = 0)
-* int LincomEntry::SetOffset(const char* scale, int index = 0)
-* int LincomEntry::SetOffset(std::complex<double> offset, int index = 0)
+* int LincomEntry::SetInput(const char *field, int index)
+* int LincomEntry::SetScale(double scale, int index)
+* int LincomEntry::SetScale(const char* scale, int index)
+* int LincomEntry::SetScale(std::complex<double> scale, int index)
+* int LincomEntry::SetOffset(double offset, int index)
+* int LincomEntry::SetOffset(const char* scale, int index)
+* int LincomEntry::SetOffset(std::complex<double> offset, int index)
 
   These functions will change the specified field parameter associated with the
   input field with the given index, which should be between zero and two.  To
@@ -430,9 +542,9 @@ Defined in getdata/linterpentry.h
   This creates a new LINTERP entry object with default parameters.
 
 * LinterpEntry::LinterpEntry(const char *field_code, const char *in_field,
-    const char *table, int format_file = 0)
+    const char *table, int fragment_index = 0)
 
-* virtual const char *Entry::Input()
+* virtual const char *Entry::Input(int index = 0)
 * virtual const char *LinterpEntry::Table()
 
   These methods, re-implemented from the Entry class, return the corresponding
@@ -457,14 +569,15 @@ Defined in getdata/bitentry.h and getdata/sbitentry.h
   These creates a new BIT or SBIT entry object with default parameters.
 
 * BitEntry::BitEntry(const char *field_code, const char *in_field, int bitnum,
-    int numbits = 1, int format_file = 0)
+    int numbits = 1, int fragment_index = 0)
 * SBitEntry::SBitEntry(const char *field_code, const char *in_field, int bitnum,
-    int numbits = 1, int format_file = 0)
+    int numbits = 1, int fragment_index = 0)
 
-* virtual const char *Input(int __gd_unused index = 0)
+* virtual const char *Input(int index = 0)
 * virtual int FirstBit()
 * virtual int NumBits()
-* virtual const char *Scalar(int index = 0)
+* virtual const char *Scalar(int index)
+* virtual int ScalarIndex(int index)
 
   These methods, re-implemented from the Entry class, return the corresponding
   field parameter.
@@ -489,14 +602,14 @@ Defined in getdata/multiplyentry.h
   This creates a new MULTIPLY entry object with default parameters.
 
 * MultiplyEntry::MultiplyEntry(const char *field_code, const char *in_field1,
-    const char *in_field2, int format_file = 0)
+    const char *in_field2, int fragment_index = 0)
 
-* virtual const char *MultiplyEntry::Input(int index = 0)
+* virtual const char *MultiplyEntry::Input(int index)
 
   This method, re-implemented from the Entry class, returns one of the input
   fields.
 
-* int MultiplyEntry::SetInput(const char *field, int index = 0)
+* int MultiplyEntry::SetInput(const char *field, int index)
 
   This function will change the specified input field with the given index,
   which should be zero or one.
@@ -512,10 +625,12 @@ Defined in getdata/phaseentry.h
   This creates a new PHASE entry object with default parameters.
 
 * PhaseEntry::PhaseEntry(const char *field_code, const char *in_field,
-    int shift, int format_file = 0)
+    int shift, int fragment_index = 0)
 
-* virtual const char *PhaseEntry::Input()
+* virtual const char *PhaseEntry::Input(int index = 0)
 * virtual long int PhaseEntry::Shift()
+* virtual const char *PhasEntry::Scalar(int index = 0)
+* virtual int PhaseEntry::ScalarIndex(int index = 0)
 
   These methods, re-implemented from the Entry class, return the corresponding
   field parameter.
@@ -538,25 +653,25 @@ Defined in getdata/lincomentry.h
   This creates a new LINCOM entry object with default parameters.
 
 * PolynomEntry::PolynomEntry(const char *field_code, int poly_ord,
-    const char *in_field, double *a, int format_file = 0)
+    const char *in_field, double *a, int fragment_index = 0)
 * PolynomEntry::PolynomEntry(const char *field_code, int poly_ord,
-    const char *in_field, std::complex<double> *a int format_file = 0)
+    const char *in_field, std::complex<double> *a int fragment_index = 0)
 
 * virtual const char *PolynomEntry::Input(int index = 0)
 * virtual int PolynomEntry::ComplexScalars()
 * virtual int PolynomEntry::PolyOrd()
-* virtual double PolynomEntry::Coefficient(int index = 0)
-* virtual std::complex<double> PolynomEntry::CCoefficient(int index = 0)
-* virtual const char *PolynomEntry::Scalar(int index = 0)
-* virtual int PolynomEntry::ScalarIndex(int index = 0)
+* virtual double PolynomEntry::Coefficient(int index)
+* virtual std::complex<double> PolynomEntry::CCoefficient(int index)
+* virtual const char *PolynomEntry::Scalar(int index)
+* virtual int PolynomEntry::ScalarIndex(int index)
 
   These methods, re-implemented from the Entry class, return the corresponding
   field parameter.
 
 * int PolynomEntry::SetInput(const char *field)
-* int PolynomEntry::SetCoefficient(double scale, int index = 0)
-* int PolynomEntry::SetCoefficient(const char* scale, int index = 0)
-* int PolynomEntry::SetCoefficient(std::complex<double> scale, int index = 0)
+* int PolynomEntry::SetCoefficient(double scale, int index)
+* int PolynomEntry::SetCoefficient(const char* scale, int index)
+* int PolynomEntry::SetCoefficient(std::complex<double> scale, int index)
 
   These functions will change the specified field parameter associated with the
   input field with the given index, which should be between zero and two.  To
@@ -581,14 +696,14 @@ Defined in getdata/divideentry.h
   This creates a new DIVIDE entry object with default parameters.
 
 * DivideEntry::DivideEntry(const char *field_code, const char *in_field1,
-    const char *in_field2, int format_file = 0)
+    const char *in_field2, int fragment_index = 0)
 
-* virtual const char *DivideEntry::Input(int index = 0)
+* virtual const char *DivideEntry::Input(int index)
 
   This method, re-implemented from the Entry class, returns one of the input
   fields.
 
-* int DivideEntry::SetInput(const char *field, int index = 0)
+* int DivideEntry::SetInput(const char *field, int index)
 
   This function will change the specified input field with the given index,
   which should be zero or one.
@@ -604,27 +719,89 @@ Defined in getdata/recipentry.h
   This creates a new RECIP entry object with default parameters.
 
 * RecipEntry::RecipEntry(const char *field_code, const char *in_field1,
-    const char *in_field2, int format_file = 0)
+    const char *in_field2, int fragment_index = 0)
 
-* virtual const char *RecipEntry::Input()
+* virtual const char *RecipEntry::Input(int index = 0)
 * virtual int RecipEntry::ComplexScalars()
 * virtual double RecipEntry::Dividend()
 * virtual std::complex<double> RecipEntry::CDividend()
 * virtual const char *RecipEntry::Scalar()
-* virtual int RecipEntry::ScalarIndex()
+* virtual int RecipEntry::ScalarIndex(int index = 0)
 
   These methods, re-implemented from the Entry class, return the corresponding
   field parameter.
 
 * int RecipEntry::SetInput(const char *field)
-* virtual std::complex<double> CDividend()
-* int SetInput(const char* field)
-* int SetDividend(double coeff)
-* int SetDividend(const char* coeff)
-* int SetDividend(std::complex<double> coeff)
+* int RecipEntry::SetDividend(double coeff)
+* int RecipEntry::SetDividend(const char* coeff)
+* int RecipEntry::SetDividend(std::complex<double> coeff)
 
-  This function will change the specified input field with the given index,
-  which should be zero or one.
+  These functions will change the specified input field or the field dividend
+  to the the value or field code supplied.
+
+
+WindowEntry Class
+-------------------
+
+Defined in getdata/windowentry.h
+
+* WindowEntry::WindowEntry()
+  
+  This creates a new WINDOW entry object with default parameters.
+
+* WindowEntry::WindowEntry(const char *field_code, const char *in_field,
+    const char *check_field, int fragment_index = 0)
+
+* virtual const char *WindowEntry::Input(int index)
+* virtual const char *WindowEntry::Scalar(int index = 0)
+* virtual int WindowEntry::ScalarIndex(int index = 0)
+* virtual WindOpType WindowEntry::WindOp()
+* virtual gd_triplet_t WindowEntry::Threshold()
+
+  These methods, re-implemented from the Entry class, return the corresponding
+  field parameter.
+
+* int WindowEntry::SetInput(const char *field, int index)
+* int WindowEntry::SetWindOp(WindOpType coeff)
+* int WindowEntry::SetThreshold(gd_triplet_t threshold)
+* int WindowEntry::SetThreshold(const char *threhsold)
+
+  These functions will change the specified input field, check field, or the
+  window operation or threshold to the the value or field code supplied.  For
+  allowed values of variables of type WindOpType, see the CONSTANTS section
+  above.
+
+
+MplexEntry Class
+-------------------
+
+Defined in getdata/mplex.h
+
+* Mplex::MplexEntry()
+  
+  This creates a new MPLEX entry object with default parameters.
+
+* MplexEntry::MplexEntry(const char *field_code, const char *in_field,
+    const char *check_field, int fragment_index = 0)
+
+* virtual const char *MplexEntry::Input(int index)
+* virtual const char *MplexEntry::Scalar(int index)
+* virtual int MplexEntry::ScalarIndex(int index)
+* virtual int MplexEntry::CountVal()
+* virtual int MplexEntry::Period()
+
+  These methods, re-implemented from the Entry class, return the corresponding
+  field parameter.
+
+* int MplexEntry::SetInput(const char *field, int index)
+* int MplexEntry::SetWindOp(WindOpType coeff)
+* int MplexEntry::SetCountVal(int count_val)
+* int MplexEntry::SetCountVal(const char *count_val)
+* int MplexEntry::SetPeriod(int period)
+* int MplexEntry::SetPeriod(const char *period)
+
+  These functions will change the specified input field, check field, or the
+  count value or max to the value or field code supplied.
 
 
 CarraytEntry Class
@@ -637,7 +814,7 @@ Defined in getdata/constentry.h
   This creates a new CONST entry object with default parameters.
 
 * CarraytEntry::CarraytEntry(const char *field_code, DataType type,
-    int format_file = 0)
+    int fragment_index = 0)
 
 * virtual DataType CarraytEntry::ConstType()
 * virtual size_t Carray::ArrayLen()
@@ -662,7 +839,7 @@ Defined in getdata/constentry.h
   This creates a new CONST entry object with default parameters.
 
 * ConstEntry::ConstEntry(const char *field_code, DataType type,
-    int format_file = 0)
+    int fragment_index = 0)
 
 * virtual DataType ConstEntry::ConstType()
 
@@ -684,7 +861,7 @@ Defined in getdata/stringentry.h
   
   This creates a new STRING entry object with default parameters.
 
-* StringEntry::StringEntry(const char *field_code, int format_file = 0)
+* StringEntry::StringEntry(const char *field_code, int fragment_index = 0)
 
 
 IndexEntry Class
