@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2010 D. V. Wiebe
+ * Copyright (C) 2005-2015 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -21,17 +21,9 @@
  */
 #include "internal.h"
 
-#ifdef STDC_HEADERS
-#include <inttypes.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-#endif
-
-#ifdef HAVE_LIBGEN_H
-#include <libgen.h>
+/* This is defined on BSD */
+#ifndef MAXSYMLINKS
+#define MAXSYMLINKS 20
 #endif
 
 #ifdef GETDATA_DEBUG
@@ -39,11 +31,16 @@ int gd_col_count = 0;
 char gd_debug_col[GD_COL_SIZE + 1] = "";
 #endif
 
+int _GD_EntryCmp(const void *a, const void *b)
+{
+  return strcmp((*(gd_entry_t**)a)->field, (*(gd_entry_t**)b)->field);
+}
+
 /* _GD_GetLine: read non-comment line from format file.  The line is newly
  *      malloc'd.  Returns a pointer if successful, NULL if unsuccessful.
  *      The length read is provided in *n.  Increments *linenum as appropriate;
  */
-char *_GD_GetLine(FILE *fp, size_t *n, int* linenum)
+char *_GD_GetLine(FILE *restrict fp, size_t *restrict n, int *restrict linenum)
 {
   ssize_t len;
 
@@ -62,7 +59,7 @@ char *_GD_GetLine(FILE *fp, size_t *n, int* linenum)
 
 
   if (len != -1) {
-    dreturn("\"%s\" (%zu)", line, *n);
+    dreturn("\"%s\" (%" PRNsize_t ")", line, *n);
     return line; /* a line was read */
   }
 
@@ -99,13 +96,17 @@ gd_type_t _GD_LegacyType(char c)
 }
 
 /* Binary search to find the field */
-gd_entry_t* _GD_FindField(DIRFILE* D, const char* field_code,
-    gd_entry_t** list, unsigned int u, unsigned int *index)
+gd_entry_t *_GD_FindField(const DIRFILE *restrict D,
+    const char *restrict field_code, gd_entry_t *const *list, unsigned int u,
+    int dealias, unsigned int *restrict index)
 {
   int c;
+  char *ptr;
+  gd_entry_t *E = NULL;
   unsigned int i, l = 0;
+  const unsigned int ou = u;
 
-  dtrace("%p, \"%s\", %p, %u, %p", D, field_code, list, u, index);
+  dtrace("%p, \"%s\", %p, %u, %i, %p", D, field_code, list, u, dealias, index);
 
   /* handle FILEFRAM */
   if (D->standards < 6 && (D->flags & GD_PEDANTIC) &&
@@ -120,23 +121,53 @@ gd_entry_t* _GD_FindField(DIRFILE* D, const char* field_code,
     else if (c > 0)
       l = i + 1;
     else {
-      if (index != NULL) 
+      E = list[i];
+      if (dealias && E && E->field_type == GD_ALIAS_ENTRY)
+        E = E->e->entry[0];
+
+      if (index != NULL)
         *index = i;
 
-      dreturn("%p", list[i]);
-      return list[i];
+      dreturn("%p", E);
+      return E;
     }
   }
 
-  if (index != NULL) 
+  if (index != NULL)
     *index = u;
 
-  dreturn("%p", NULL);
-  return NULL;
+  /* not found perhaps it's an subfield of an aliased field? */
+  if ((ptr = (char*)strchr(field_code, '/'))) {
+    char *new_code = strdup(field_code);
+    if (new_code) {
+      new_code[ptr - field_code] = '\0';
+      E = _GD_FindField(D, new_code, list, ou, 0, NULL);
+      free(new_code);
+
+      if (E && E->field_type == GD_ALIAS_ENTRY && E->e->entry[0]) {
+        size_t plen = strlen(E->e->entry[0]->field);
+        new_code = (char*)malloc(plen + strlen(ptr) + 2);
+        if (new_code) {
+          strcpy(new_code, E->e->entry[0]->field);
+          new_code[plen] = '/';
+          strcpy(new_code + plen + 1, ptr + 1);
+
+          E = _GD_FindField(D, new_code, list, ou, 1, NULL);
+
+          free(new_code);
+        }
+      } else
+        E = NULL;
+    }
+  }
+
+  dreturn("%p", E);
+  return E;
 }
 
 /* Insertion sort the entry list */
-void _GD_InsertSort(DIRFILE* D, gd_entry_t* E, int u) gd_nothrow
+void _GD_InsertSort(DIRFILE *restrict D, gd_entry_t *restrict E, int u)
+  gd_nothrow
 {
   dtrace("%p, %p, %i", D, E, u);
 
@@ -154,7 +185,7 @@ void* _GD_Alloc(DIRFILE* D, gd_type_t type, size_t n)
 {
   void* ptr = NULL;
 
-  dtrace("%p, 0x%x, %zu", D, type, n);
+  dtrace("%p, 0x%x, %" PRNsize_t, D, type, n);
   if (n == 0) {
     _GD_InternalError(D);
     dreturn("%p", NULL);
@@ -172,43 +203,37 @@ void* _GD_Alloc(DIRFILE* D, gd_type_t type, size_t n)
     return NULL;
   }
 
-  ptr = malloc(n * GD_SIZE(type));
-
-  if (ptr == NULL)
-    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+  ptr = _GD_Malloc(D, n * GD_SIZE(type));
 
   dreturn("%p", ptr);
   return ptr;
 }
 
 /* compute LUT table path -- this is used by _GD_Change, so e may not be E->e */
-int _GD_SetTablePath(DIRFILE *D, gd_entry_t *E, struct _gd_private_entry *e)
+int _GD_SetTablePath(DIRFILE *restrict D, const gd_entry_t *restrict E,
+    struct gd_private_entry_ *restrict e)
 {
   char *temp_buffer;
 
   dtrace("%p, %p, %p", D, E, e);
 
-  if (E->EN(linterp,table)[0] == '/') {
-    e->u.linterp.table_path = strdup(E->EN(linterp,table));
-    if (e->u.linterp.table_path == NULL) {
-      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-      dreturn("%i", 1);
-      return 1;
-    }
-  } else {
-    e->u.linterp.table_path = (char *)malloc(strlen(E->EN(linterp,table)) + 2 +
-        strlen(D->fragment[E->fragment_index].cname));
-    if (e->u.linterp.table_path == NULL) {
-      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-      dreturn("%i", 1);
-      return 1;
-    }
-    temp_buffer = strdup(D->fragment[E->fragment_index].cname);
-    strcpy(e->u.linterp.table_path, dirname(temp_buffer));
-    strcat(e->u.linterp.table_path, "/");
-    strcat(e->u.linterp.table_path, E->EN(linterp,table));
-    free(temp_buffer);
+  e->u.linterp.table_dirfd = _GD_GrabDir(D,
+      D->fragment[E->fragment_index].dirfd, E->EN(linterp,table), 0);
+
+  temp_buffer = _GD_Strdup(D, E->EN(linterp,table));
+  if (temp_buffer == NULL) {
+    _GD_ReleaseDir(D, e->u.linterp.table_dirfd);
+    dreturn("%i", 1);
+    return 1;
   }
+  e->u.linterp.table_file = _GD_Strdup(D, basename(temp_buffer));
+  if (e->u.linterp.table_file == NULL) {
+    _GD_ReleaseDir(D, e->u.linterp.table_dirfd);
+    free(temp_buffer);
+    dreturn("%i", 1);
+    return 1;
+  }
+  free(temp_buffer);
 
   dreturn("%i", 0);
   return 0;
@@ -217,78 +242,94 @@ int _GD_SetTablePath(DIRFILE *D, gd_entry_t *E, struct _gd_private_entry *e)
 /* LUT comparison function for qsort */
 static int lutcmp(const void* a, const void* b)
 {
-  double dx = ((struct _gd_lut *)a)->x - ((struct _gd_lut *)b)->x;
+  double dx = ((struct gd_lut_ *)a)->x - ((struct gd_lut_ *)b)->x;
   return (dx < 0) ? -1 : (dx > 0) ? 1 : 0;
 }
 
 /* _GD_ReadLinterpFile: Read in the linterp data for this field
 */
-void _GD_ReadLinterpFile(DIRFILE* D, gd_entry_t *E)
+void _GD_ReadLinterpFile(DIRFILE *restrict D, gd_entry_t *restrict E)
 {
   FILE *fp;
-  struct _gd_lut *ptr;
-  int i;
+  struct gd_lut_ *ptr;
+  int i, fd;
   int dir = -1;
-  char *line;
+  char *line = NULL;
   size_t n = 0;
   int linenum = 0;
   double yr, yi;
-  int buf_len = 100;
+  int buf_len = GD_LUT_CHUNK;
 
   dtrace("%p, %p", D, E);
 
-  if (E->e->u.linterp.table_path == NULL)
+  if (E->e->u.linterp.table_file == NULL)
     if (_GD_SetTablePath(D, E, E->e)) {
       dreturnvoid();
       return;
     }
 
-  fp = fopen(E->e->u.linterp.table_path, "r");
-  if (fp == NULL) {
-    _GD_SetError(D, GD_E_OPEN_LINFILE, GD_E_LINFILE_OPEN, NULL, 0,
-        E->e->u.linterp.table_path);
+  fd = gd_OpenAt(D, E->e->u.linterp.table_dirfd, E->e->u.linterp.table_file,
+      O_RDONLY, 0666);
+  if (fd == -1) {
+    _GD_SetError(D, GD_E_IO, GD_E_IO_OPEN, E->EN(linterp,table), 0, NULL);
     dreturnvoid();
     return;
   }
 
+  fp = fdopen(fd, "rb");
+  if (fp == NULL) {
+    _GD_SetError(D, GD_E_IO, GD_E_IO_OPEN, E->EN(linterp,table), 0, NULL);
+    dreturnvoid();
+    return;
+  }
+
+  E->e->u.linterp.lut = _GD_Malloc(D, buf_len * sizeof(*E->e->u.linterp.lut));
+
+  if (E->e->u.linterp.lut == NULL)
+    goto LUT_ERROR;
+
   /* read the first line to see whether the table is complex valued */
   if ((line = _GD_GetLine(fp, &n, &linenum))) {
     char ystr[50];
     if (sscanf(line, "%lg %49s", &yr, ystr) == 2)
       E->e->u.linterp.complex_table = (strchr(ystr, ';') == NULL) ? 0 : 1;
+    else {
+      _GD_SetError(D, GD_E_LUT, GD_E_LUT_SYNTAX, E->EN(linterp,table), linenum,
+          NULL);
+      goto LUT_ERROR;
+    }
   } else {
     if (errno == EOVERFLOW)
       /* line too long */
-      _GD_SetError(D, GD_E_LINE_TOO_LONG, 0, E->e->u.linterp.table_path,
-          linenum, NULL);
+      _GD_SetError(D, GD_E_LINE_TOO_LONG, 0, E->EN(linterp,table), linenum,
+          NULL);
     else 
       /* no data in file! */
-      _GD_SetError(D, GD_E_OPEN_LINFILE, GD_E_LINFILE_LENGTH, NULL, 0,
-          E->e->u.linterp.table_path);
-    fclose(fp);
-    dreturnvoid();
-    return;
-  }
-
-  E->e->u.linterp.lut = (struct _gd_lut *)malloc(buf_len *
-      sizeof(struct _gd_lut));
-
-  if (E->e->u.linterp.lut == NULL) {
-    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-    fclose(fp);
-    dreturnvoid();
-    return;
+      _GD_SetError(D, GD_E_LUT, GD_E_LUT_LENGTH, E->EN(linterp,table), 0, NULL);
+    goto LUT_ERROR;
   }
 
   /* now read in the data -- we've already read line one */
   i = 0;
   do {
     if (E->e->u.linterp.complex_table) {
-      sscanf(line, "%lg %lg;%lg", &(E->e->u.linterp.lut[i].x), &yr, &yi);
-      _gd_l2c(E->e->u.linterp.lut[i].y.c, yr, yi);
+      if (sscanf(line, "%lg %lg;%lg", &(E->e->u.linterp.lut[i].x), &yr, &yi) ==
+          3)
+      {
+        gd_li2cs_(E->e->u.linterp.lut[i].y.c, yr, yi);
+      } else {
+        _GD_SetError(D, GD_E_LUT, GD_E_LUT_SYNTAX, E->EN(linterp,table),
+            linenum, NULL);
+        goto LUT_ERROR;
+      }
     } else
-      sscanf(line, "%lg %lg", &(E->e->u.linterp.lut[i].x),
-          &(E->e->u.linterp.lut[i].y.r));
+      if (sscanf(line, "%lg %lg", &(E->e->u.linterp.lut[i].x),
+            &(E->e->u.linterp.lut[i].y.r)) != 2)
+      {
+        _GD_SetError(D, GD_E_LUT, GD_E_LUT_SYNTAX, E->EN(linterp,table),
+            linenum, NULL);
+        goto LUT_ERROR;
+      }
 
     if (dir > -2 && i > 0 && E->e->u.linterp.lut[i].x !=
         E->e->u.linterp.lut[i - 1].x)
@@ -301,17 +342,11 @@ void _GD_ReadLinterpFile(DIRFILE* D, gd_entry_t *E)
 
     i++;
     if (i >= buf_len) {
-      buf_len += 100;
-      ptr = (struct _gd_lut *)realloc(E->e->u.linterp.lut, buf_len *
-          sizeof(struct _gd_lut));
+      buf_len += GD_LUT_CHUNK;
+      ptr = _GD_Realloc(D, E->e->u.linterp.lut, buf_len * sizeof(*ptr));
 
-      if (ptr == NULL) {
-        free(E->e->u.linterp.lut);
-        _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-        fclose(fp);
-        dreturnvoid();
-        return;
-      }
+      if (ptr == NULL)
+        goto LUT_ERROR;
 
       E->e->u.linterp.lut = ptr;
     }
@@ -319,25 +354,15 @@ void _GD_ReadLinterpFile(DIRFILE* D, gd_entry_t *E)
   } while ((line = _GD_GetLine(fp, &n, &linenum)));
 
   if (i < 2) {
-    free(E->e->u.linterp.lut);
-    _GD_SetError(D, GD_E_OPEN_LINFILE, GD_E_LINFILE_LENGTH, NULL, 0,
-        E->e->u.linterp.table_path);
-    fclose(fp);
-    dreturnvoid();
-    return;
+    _GD_SetError(D, GD_E_LUT, GD_E_LUT_LENGTH, NULL, 0, E->EN(linterp,table));
+    goto LUT_ERROR;
   }
 
   /* Free unused memory */
-  ptr = (struct _gd_lut *)realloc(E->e->u.linterp.lut, i
-      * sizeof(struct _gd_lut));
+  ptr = _GD_Realloc(D, E->e->u.linterp.lut, i * sizeof(*ptr));
 
-  if (ptr == NULL) {
-    free(E->e->u.linterp.lut);
-    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-    fclose(fp);
-    dreturnvoid();
-    return;
-  }
+  if (ptr == NULL)
+    goto LUT_ERROR;
 
   E->e->u.linterp.table_monotonic = -1;
   E->e->u.linterp.lut = ptr;
@@ -345,18 +370,26 @@ void _GD_ReadLinterpFile(DIRFILE* D, gd_entry_t *E)
 
   /* sort the LUT */
   if (dir == -2)
-    qsort(E->e->u.linterp.lut, i, sizeof(struct _gd_lut), lutcmp);
+    qsort(E->e->u.linterp.lut, i, sizeof(struct gd_lut_), lutcmp);
 
   fclose(fp);
   dreturnvoid();
+  return;
+
+LUT_ERROR:
+  free(E->e->u.linterp.lut);
+  E->e->u.linterp.lut = NULL;
+  free(line);
+  fclose(fp);
+  dreturnvoid();
 }
 
 /* _GD_GetIndex: get LUT index.
 */
-static size_t _GD_GetIndex(double x, const struct _gd_lut *lut, size_t idx,
+static size_t _GD_GetIndex(double x, const struct gd_lut_ *lut, size_t idx,
     size_t n)
 {
-  dtrace("%g, %p, %zu, %zu", x, lut, idx, n);
+  dtrace("%g, %p, %" PRNsize_t ", %" PRNsize_t, x, lut, idx, n);
 
   /* Just linearly search - we're probably right to start    */
   /* increment until we are bigger */
@@ -367,7 +400,7 @@ static size_t _GD_GetIndex(double x, const struct _gd_lut *lut, size_t idx,
   while ((idx > 0) && (x < lut[idx].x))
     idx--;
 
-  dreturn("%zu", idx);
+  dreturn("%" PRNsize_t, idx);
   return idx;
 }
 
@@ -441,15 +474,16 @@ static size_t _GD_GetIndex(double x, const struct _gd_lut *lut, size_t idx,
 
 /* _GD_LinterpData: calibrate data using lookup table lut
 */
-void _GD_LinterpData(DIRFILE* D, void *data, gd_type_t type, int complex_table,
-    const double *data_in, size_t npts, const struct _gd_lut *lut, size_t n_ln)
+void _GD_LinterpData(DIRFILE *restrict D, void *restrict data, gd_type_t type,
+    int complex_table, const double *restrict data_in, size_t npts,
+    const struct gd_lut_ *restrict lut, size_t n_ln)
 {
   int idx = 0;
   size_t i;
   double x;
 
-  dtrace("%p, %p, 0x%x, %i, %p, %zu, %p, %zu", D, data, type, complex_table,
-      data_in, npts, lut, n_ln);
+  dtrace("%p, %p, 0x%x, %i, %p, %" PRNsize_t ", %p, %" PRNsize_t, D, data, type,
+      complex_table, data_in, npts, lut, n_ln);
 
   switch (type) {
     case GD_NULL:                          break;
@@ -535,14 +569,15 @@ void _GD_LinterpData(DIRFILE* D, void *data, gd_type_t type, int complex_table,
   }
 
 /* Compute a lincom, all at once */
-void _GD_LincomData(DIRFILE* D, int n, void* data1, gd_type_t return_type,
-    double *data2, double *data3, double* m, double *b, gd_spf_t *spf,
-    size_t n_read)
+void _GD_LincomData(DIRFILE *restrict D, int n, void *restrict data1,
+    gd_type_t return_type, const double *restrict data2,
+    const double *restrict data3, const double *restrict m,
+    const double *restrict b, const unsigned int *restrict spf, size_t n_read)
 {
   size_t i;
 
-  dtrace("%p, %i, %p, 0x%x, %p, %p, %p, %p, %p, %zu", D, n, data1, return_type,
-      data2, data3, m, b, spf, n_read);
+  dtrace("%p, %i, %p, 0x%x, %p, %p, %p, %p, %p, %" PRNsize_t, D, n, data1,
+      return_type, data2, data3, m, b, spf, n_read);
 
   switch(return_type) {
     case GD_NULL:                         break;
@@ -643,14 +678,15 @@ void _GD_LincomData(DIRFILE* D, int n, void* data1, gd_type_t return_type,
 #endif
 
 /* Compute a complex valued lincom, all at once */
-void _GD_CLincomData(DIRFILE* D, int n, void* data1, gd_type_t return_type,
-    GD_DCOMPLEXP(data2), GD_DCOMPLEXP(data3), GD_DCOMPLEXV(m), GD_DCOMPLEXV(b),
-    gd_spf_t *spf, size_t n_read)
+void _GD_CLincomData(DIRFILE *restrict D, int n, void *restrict data1,
+    gd_type_t return_type, const GD_DCOMPLEXP_t restrict data2,
+    const GD_DCOMPLEXP_t restrict data3, GD_DCOMPLEXV(m), GD_DCOMPLEXV(b),
+    const unsigned int *restrict spf, size_t n_read)
 {
   size_t i;
 
-  dtrace("%p, %i, %p, 0x%x, %p, %p, %p, %p, %p, %zu", D, n, data1, return_type,
-      data2, data3, m, b, spf, n_read);
+  dtrace("%p, %i, %p, 0x%x, %p, %p, %p, %p, %p, %" PRNsize_t, D, n, data1,
+      return_type, data2, data3, m, b, spf, n_read);
 
   switch(return_type) {
     case GD_NULL:                         break;
@@ -677,7 +713,7 @@ void _GD_CLincomData(DIRFILE* D, int n, void* data1, gd_type_t return_type,
   do { \
     for (i = 0; i < n_read; i++) { \
       const t d = ((t *)data)[2 * i] * ((t *)data)[2 * i] + \
-        ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
+      ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
       ((t *)data)[2 * i] = (t)(dividend * ((t *)data)[2 * i] / d); \
       ((t *)data)[2 * i + 1] = (t)(-dividend * ((t *)data)[2 * i + 1] / d); \
     } \
@@ -690,12 +726,13 @@ void _GD_CLincomData(DIRFILE* D, int n, void* data1, gd_type_t return_type,
                            ((t *)data)[i] = (t)(dividend / ((t *)data)[i])
 
 /* Invert a vector */
-void _GD_InvertData(DIRFILE* D, void* data, gd_type_t return_type,
-    double dividend, size_t n_read)
+void _GD_InvertData(DIRFILE *restrict D, void *restrict data,
+    gd_type_t return_type, double dividend, size_t n_read)
 {
   size_t i;
 
-  dtrace("%p, %p, 0x%x, %g, %zu", D, data, return_type, dividend, n_read);
+  dtrace("%p, %p, 0x%x, %g, %" PRNsize_t, D, data, return_type, dividend,
+      n_read);
 
   switch(return_type) {
     case GD_NULL:                         break;
@@ -725,8 +762,8 @@ void _GD_InvertData(DIRFILE* D, void* data, gd_type_t return_type,
   do { \
     for (i = 0; i < n_read; i++) { \
       const t d = ((t *)data)[2 * i] * ((t *)data)[2 * i] + \
-        ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
-      ((t *)data)[2 * i] = (t)((dividend[0] * ((t *)data)[2 * i] - \
+      ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
+      ((t *)data)[2 * i] = (t)((dividend[0] * ((t *)data)[2 * i] + \
           dividend[1] * ((t *)data)[2 * i + 1]) / d); \
       ((t *)data)[2 * i + 1] = (t)((dividend[1] * ((t *)data)[2 * i] - \
           dividend[0] * ((t *)data)[2 * i]) / d); \
@@ -737,7 +774,7 @@ void _GD_InvertData(DIRFILE* D, void* data, gd_type_t return_type,
   do { \
     for (i = 0; i < n_read; i++) { \
       const t d = ((t *)data)[2 * i] * ((t *)data)[2 * i] + \
-        ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
+      ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
       ((t *)data)[2 * i] = (t)((dividend[0] * ((t *)data)[2 * i] - \
           dividend[1] * ((t *)data)[2 * i + 1]) / d); \
     } \
@@ -746,13 +783,13 @@ void _GD_InvertData(DIRFILE* D, void* data, gd_type_t return_type,
 #endif
 
 /* Invert a vector */
-void _GD_CInvertData(DIRFILE* D, void* data, gd_type_t return_type,
-    GD_DCOMPLEXA(dividend), size_t n_read)
+void _GD_CInvertData(DIRFILE *restrict D, void *restrict data,
+    gd_type_t return_type, GD_DCOMPLEXA(dividend), size_t n_read)
 {
   size_t i;
 
-  dtrace("%p, %p, 0x%x, %g;%g, %zu", D, data, return_type, creal(dividend),
-      cimag(dividend), n_read);
+  dtrace("%p, %p, 0x%x, %g;%g, %" PRNsize_t, D, data, return_type,
+      creal(dividend), cimag(dividend), n_read);
 
   switch(return_type) {
     case GD_NULL:                         break;
@@ -774,14 +811,14 @@ void _GD_CInvertData(DIRFILE* D, void* data, gd_type_t return_type,
   dreturnvoid();
 }
 
-int _GD_GetRepr(DIRFILE* D, const char* field_code_in, char** field_code)
+int _GD_GetRepr(DIRFILE *restrict D, const char *restrict field_code_in,
+    char **restrict field_code, int err)
 {
   int repr = GD_REPR_NONE;
   const int field_code_len = strlen(field_code_in);
 
-  dtrace("%p, \"%s\", %p", D, field_code_in, field_code);
+  dtrace("%p, \"%s\", %p, %i", D, field_code_in, field_code, err);
 
-  *field_code = (char *)field_code_in;
   /* find the representation, if any */
   if (field_code_len > 2 && field_code_in[field_code_len - 2] == '.') {
     switch (field_code_in[field_code_len - 1]) {
@@ -798,36 +835,36 @@ int _GD_GetRepr(DIRFILE* D, const char* field_code_in, char** field_code)
         repr = GD_REPR_ARG;
         break;
       default:
-        _GD_SetError(D, GD_E_BAD_REPR, GD_E_REPR_UNKNOWN, NULL, 0,
-            field_code_in + field_code_len - 1);
+        if (err)
+          _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0,
+              field_code_in);
         dreturn("%i", 0);
         return 0;
     }
 
     /* make a copy of the field code without the representation */
-    *field_code = strdup(field_code_in);
-    if (*field_code == NULL) 
-      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-    else
+    *field_code = _GD_Strdup(D, field_code_in);
+    if (*field_code)
       (*field_code)[field_code_len - 2] = '\0';
-  }
+  } else
+    *field_code = (char *)field_code_in;
 
   dreturn("%i", repr);
   return repr;
 }
 
 /* Ensure that an input field has been identified (with error checking) */
-int _GD_BadInput(DIRFILE* D, gd_entry_t* E, int i)
+int _GD_BadInput(DIRFILE *D, const gd_entry_t *E, int i, gd_entype_t t, int err)
 {
-  char* code;
+  char *code;
 
-  dtrace("%p, %p, %i", D, E, i);
+  dtrace("%p, %p, %i, 0x%X, %i", D, E, i, t, err);
 
   if (E->e->entry[i] == NULL) {
     E->e->entry[i] = _GD_FindFieldAndRepr(D, E->in_fields[i], &code,
-        &E->e->repr[i], NULL, 1);
+        &E->e->repr[i], NULL, 1, err);
 
-    if (D->error) {
+    if (E->e->entry[i] == NULL) {
       dreturn("%i", 1);
       return 1;
     }
@@ -836,9 +873,17 @@ int _GD_BadInput(DIRFILE* D, gd_entry_t* E, int i)
       free(code);
   }
 
-  /* scalar entries not allowed */
-  if (E->e->entry[i]->field_type & GD_SCALAR_ENTRY) {
-    _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_FORMAT, E->field, 0,
+  /* check field type */
+  if (t == GD_NO_ENTRY) {
+    /* scalar entries not allowed */
+    if (E->e->entry[i]->field_type & GD_SCALAR_ENTRY_BIT) {
+      _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_FORMAT, E->field, 0,
+          E->e->entry[i]->field);
+      dreturn("%i", 1);
+      return 1;
+    }
+  } else if (E->e->entry[i]->field_type != t) {
+    _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_FORMAT, E->field, 0,
         E->e->entry[i]->field);
     dreturn("%i", 1);
     return 1;
@@ -848,19 +893,43 @@ int _GD_BadInput(DIRFILE* D, gd_entry_t* E, int i)
   return 0;
 }
 
-/* Find the entry and the representation */
-gd_entry_t* _GD_FindFieldAndRepr(DIRFILE* D, const char* field_code_in,
-    char** field_code, int* repr, unsigned int *index, int set)
+/* Find an entry without a representation */
+gd_entry_t *_GD_FindEntry(DIRFILE *restrict D, const char *restrict field_code,
+    unsigned int *restrict index, int set, int err)
 {
-  gd_entry_t* E = NULL;
+  gd_entry_t *E = NULL;
 
-  dtrace("%p, \"%s\", %p, %p, %p, %i", D, field_code_in, field_code, repr,
-      index, set);
+  dtrace("%p, \"%s\", %p, %i, %i", D, field_code, index, set, err);
 
-  E = _GD_FindField(D, field_code_in, D->dot_list, D->n_dot, NULL);
+  if (D->n_dot > 0)
+    E = _GD_FindField(D, field_code, D->dot_list, D->n_dot, 1, NULL);
+
+  if (E == NULL || index != NULL)
+    E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, index);
+
+  if (E == NULL && set)
+    if (err)
+      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+
+  dreturn("%p", E);
+  return E;
+}
+
+/* Find the entry and the representation */
+gd_entry_t *_GD_FindFieldAndRepr(DIRFILE *restrict D,
+    const char *restrict field_code_in, char **restrict field_code,
+    int *restrict repr, unsigned int *restrict index, int set, int err)
+{
+  gd_entry_t *E = NULL;
+
+  dtrace("%p, \"%s\", %p, %p, %p, %i, %i", D, field_code_in, field_code, repr,
+      index, set, err);
+
+  if (D->n_dot > 0)
+    E = _GD_FindField(D, field_code_in, D->dot_list, D->n_dot, 1, NULL);
 
   if (E == NULL) {
-    *repr = _GD_GetRepr(D, field_code_in, field_code);
+    *repr = _GD_GetRepr(D, field_code_in, field_code, err);
 
     if (D->error) {
       dreturn("%p", NULL);
@@ -872,10 +941,11 @@ gd_entry_t* _GD_FindFieldAndRepr(DIRFILE* D, const char* field_code_in,
   }
 
   if (E == NULL || index != NULL)
-    E = _GD_FindField(D, *field_code, D->entry, D->n_entries, index);
+    E = _GD_FindField(D, *field_code, D->entry, D->n_entries, 1, index);
 
   if (E == NULL && set) {
-    _GD_SetError(D, GD_E_BAD_CODE, 0, NULL, 0, field_code_in);
+    if (err)
+      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code_in);
     if (field_code_in != *field_code)
       free(*field_code);
   }
@@ -883,5 +953,507 @@ gd_entry_t* _GD_FindFieldAndRepr(DIRFILE* D, const char* field_code_in,
   dreturn("%p %i", E, *repr);
   return E;
 }
+
+const char *_GD_DirName(const DIRFILE *D, int dirfd)
+{
+#ifndef GD_NO_DIR_OPEN
+  unsigned int i;
+#endif
+
+  dtrace("%p, %i", D, dirfd);
+
+#ifdef GD_NO_DIR_OPEN
+  /* in the non-POSIX case, dirfd is just the index in the dir list */
+  if (D->ndir > 0) {
+    dreturn("\"%s\"", D->dir[dirfd].path);
+    return D->dir[dirfd].path;
+  }
+#else
+  for (i = 0; i < D->ndir; ++i)
+    if (dirfd == D->dir[i].fd) {
+      dreturn("\"%s\"", D->dir[i].path);
+      return D->dir[i].path;
+    }
+#endif
+
+  /* we only get here in the early stages of opening a dirfile */
+  dreturn("\"%s\"", D->name);
+  return D->name;
+}
+
+/* This is mostly the POSIX.1 realpath(3) function, but with some
+ * optimisation due to the fact that we know that the front part of the
+ * path (car) has already been canonicalised */
+char *_GD_CanonicalPath(const char *car, const char *cdr)
+{
+  int last_element = 0;
+#if defined HAVE_READLINK && defined HAVE_LSTAT64
+  int loop_count = 0;
+#endif
+  size_t res_len, res_root, res_size, len;
+  char *res = NULL, *ptr, *work, *cur, *end;
+  dtrace("\"%s\", \"%s\"", car, cdr);
+
+  if (car && !_GD_AbsPath(cdr)) {
+    if (!_GD_AbsPath(car)) {
+      /* car is not abosulte -- don't bother trying to do anything fancy */
+      res = (char*)malloc(strlen(car) + strlen(cdr) + 2);
+      if (res == NULL) {
+        dreturn("%p", NULL);
+        return NULL;
+      }
+      sprintf(res, "%s%c%s", car, GD_DIRSEP, cdr);
+      dreturn("%s", res);
+      return res;
+    }
+    /* car is nonnull and cdr is not absolute: copy car into res and
+     * cdr into work */
+    res = strdup(car);
+    if (res == NULL) {
+      dreturn("%p", NULL);
+      return NULL;
+    }
+    res_size = (res_len = strlen(car)) + 1;
+    res_root = _GD_RootLen(res);
+
+    cur = work = strdup(cdr);
+    if (work == NULL) {
+      free(res);
+      dreturn("%p", NULL);
+      return NULL;
+    }
+  } else if (_GD_AbsPath(cdr)) {
+    /* cdr is absolute: make res "/" and copy cdr relative to / into work;
+     * ignore car */
+    res = (char*)malloc(res_size = PATH_MAX);
+    if (res == NULL) {
+      dreturn("%p", NULL);
+      return NULL;
+    }
+    _GD_Root(res, cdr, res_root);
+    res_len = res_root;
+
+    if (cdr[res_len] == '\0') {
+      dreturn("\"%s\"", res);
+      return res;
+    }
+
+    cur = work = strdup(cdr + res_len);
+    if (work == NULL) {
+      free(res);
+      dreturn("%p", NULL);
+      return NULL;
+    }
+  } else {
+    /* car is not present and cdr is relative: try to get the CWD; and make
+     * work CWD/cdr, relative to / if getcwd returned an absolute path.  If
+     * getcwd fails, just use cdr, and abandon hope of an absolute path.
+     */
+    res = (char*)malloc(res_size = PATH_MAX);
+    work = (char*)malloc(PATH_MAX);
+    if (res == NULL || work == NULL) {
+      free(res);
+      dreturn("%p", NULL);
+      return NULL;
+    }
+
+    if (gd_getcwd(work, PATH_MAX) == NULL) {
+      /* if getcwd fails, we're stuck with a relative path, oh well. */
+      free(work);
+      cur = work = strdup(cdr);
+      if (work == NULL) {
+        free(res);
+        dreturn("%p", NULL);
+        return NULL;
+      }
+      res[0] = '\0';
+      res_root = res_len = 0;
+    } else {
+      if ((len = strlen(work) + 2 + strlen(cdr)) < PATH_MAX) {
+        ptr = (char*)realloc(work, len);
+        if (ptr == NULL) {
+          free(res);
+          free(work);
+          dreturn("%p", NULL);
+          return NULL;
+        }
+        work = ptr;
+      }
+      ptr = work + strlen(work);
+      *(ptr++) = GD_DIRSEP;
+      strcpy(ptr, cdr);
+
+      if (_GD_AbsPath(work)) {
+        _GD_Root(res, work, res_root);
+        res_len = res_root;
+        cur = work + res_len;
+      } else {
+        res[0] = '\0';
+        res_root = res_len = 0;
+        cur = work;
+      }
+    }
+  }
+
+  /* now step through work, building up res as appropriate */
+  for (end = cur ; !last_element; cur = end) {
+    /* look for the next GD_DIRSEP or NUL */
+    for (; *end != '\0' && !_GD_IsDirSep(*end); ++end)
+      ;
+
+    /* end of string */
+    if (*end == '\0' && cur == end)
+      break;
+    else if (*end == '\0')
+      last_element = 1;
+    else
+      *(end++) = '\0';
+
+    if (cur[0] == '\0')
+      ; /* discard consecutive DIRSEPs */
+    else if (cur[0] == '.' && cur[1] == '\0') {
+      /* discard . */
+      continue;
+    } else if (cur[0] == '.' && cur[1] == '.' && cur[2] == '\0') {
+      /* don't strip the leading GD_DIRSEP */
+      if (res_len > res_root) {
+        /* find the last GD_DIRSEP, but don't strip the leading GD_DIRSEP */
+        for(ptr = res + res_len - 1;
+            !_GD_IsDirSep(*ptr) && ptr > res + res_root; --ptr)
+          ;
+
+        /* strip the .. if possible, otherwise append it */
+        *ptr = '\0';
+        res_len = ptr - res;
+      }
+    } else {
+      /* a path element, copy it to res */
+      len = strlen(cur) + 1;
+      if (res_len + len >= res_size) {
+        ptr = (char*)realloc(res, res_size += (len < 1000) ? 1000 : len);
+        if (ptr == NULL) {
+          free(res);
+          free(work);
+          dreturn("%p", NULL);
+          return NULL;
+        }
+        res = ptr;
+      }
+      if (res_len > 1 && !_GD_IsDirSep(res[res_len - 1]))
+        res[res_len++] = GD_DIRSEP;
+      strcpy(res + res_len, cur);
+      res_len += len - 1;
+#if defined HAVE_READLINK && defined HAVE_LSTAT64
+      {
+        gd_stat64_t statbuf;
+
+        /* check if it's a symlink */
+        if (lstat64(res, &statbuf)) {
+          if (errno == ENOENT) {
+            /* the thing doesn't exist.  I guess that means we're done;
+             * copy the rest of the work buffer onto the result and call it a
+             * day. */
+            if (*end) {
+              len = strlen(end) + 1;
+              if (res_len + len >= res_size) {
+                ptr = (char*)realloc(res, res_size += len);
+                if (ptr == NULL) {
+                  free(res);
+                  free(work);
+                  dreturn("%p", NULL);
+                  return NULL;
+                }
+                res = ptr;
+              }
+              res[res_len++] = GD_DIRSEP;
+              strcpy(res + res_len, end);
+              res_len += len - 1;
+            }
+            goto _GD_CanonicalPath_DONE;
+          }
+          /* lstat error */
+          free(res);
+          free(work);
+          dreturn("%p", NULL);
+          return NULL;
+        }
+
+        if (S_ISLNK(statbuf.st_mode)) {
+          char target[PATH_MAX];
+          ssize_t slen;
+
+          /* check for symlink loop */
+          if (loop_count++ > MAXSYMLINKS) {
+            free(res);
+            free(work);
+            dreturn("%p", NULL);
+            errno = ELOOP;
+            return NULL;
+          }
+
+          /* get the link target */
+          slen = readlink(res, target, PATH_MAX - 1);
+          if (slen == -1) {
+            free(res);
+            free(work);
+            dreturn("%p", NULL);
+            return NULL;
+          }
+          target[slen] = 0;
+
+          /* now we have to start all over again */
+          ptr = target;
+          if (_GD_AbsPath(ptr)) {
+            _GD_Root(res, target, res_root);
+            res_len = res_root;
+            ptr += res_root;
+            slen -= res_root;
+          } else if (res_len > res_root) {
+            /* strip the symlink name from res */
+            char *rptr;
+            for (rptr = res + res_len - 1; !_GD_IsDirSep(*rptr); --rptr)
+              ;
+
+            /* don't strip the root /; but do strip a previous DIRSEP */
+            if ((size_t)(rptr - res) < res_root) {
+              *(rptr + 1) = '\0';
+              res_len = (rptr - res) + 1;
+            } else {
+              *rptr = '\0';
+              res_len = rptr - res;
+            }
+          }
+
+          /* now make a new work buffer of "target/remaining", asusming
+           * remaining is non-null, otherwise, just use target */
+          if (*end == '\0') {
+            last_element = 0;
+            free(work);
+            end = work = strdup(target);
+            if (work == NULL) {
+              free(res);
+              dreturn("%p", NULL);
+              return NULL;
+            }
+          } else {
+            char *new_work, slash[2] = { GD_DIRSEP, 0 };
+            len = strlen(end) + slen + 2;
+
+            if (_GD_IsDirSep(*(ptr + slen - 1))) {
+              slash[0] = '\0';
+              len--;
+            }
+
+            new_work = (char*)malloc(len);
+            if (new_work == NULL) {
+              free(res);
+              free(work);
+              dreturn("%p", NULL);
+              return NULL;
+            }
+            sprintf(new_work, "%s%s%s", ptr, slash, end);
+            free(work);
+            end = work = new_work;
+          }
+        }
+      }
+#endif
+    }
+  }
+
+#if defined HAVE_READLINK && defined HAVE_LSTAT64
+_GD_CanonicalPath_DONE:
+#endif
+  free(work);
+
+  /* trim */
+  ptr = (char*)realloc(res, res_len + 1);
+  if (ptr)
+    res = ptr;
+
+  dreturn("\"%s\"", res);
+  return res;
+}
+
+char *_GD_MakeFullPath(DIRFILE *D, int dirfd, const char *name, int seterr)
+{
+  const char *dir;
+  char *filepath;
+  dtrace("%p, %i, \"%s\", %i", D, dirfd, name, seterr);
+
+  if (dirfd >= 0) {
+    dir = _GD_DirName(D, dirfd);
+    if (dir == NULL) {
+      dreturn("%p", NULL);
+      return NULL;
+    }
+  } else
+    dir = NULL;
+
+  filepath = _GD_CanonicalPath(dir, name);
+
+  if (seterr && filepath == NULL) {
+    if (errno == ENOMEM)
+      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+    else {
+      /* a last ditch attempt */
+      if (dir) {
+        filepath = _GD_Malloc(D, strlen(dir) + strlen(name) + 2);
+        if (filepath)
+          sprintf(filepath, "%s%c%s", dir, GD_DIRSEP, name);
+      } else 
+        filepath = _GD_Strdup(D, name);
+    }
+  }
+
+  dreturn("\"%s\"", filepath);
+  return filepath;
+}
+
+char *_GD_MakeFullPathOnly(const DIRFILE *D, int dirfd, const char *name)
+{
+  char *filepath;
+
+  dtrace("%p, %i, \"%s\"", D, dirfd, name);
+
+  /* although we cast away the constness, seterr=0 ensures nothing gets
+   * modified
+   */
+  filepath = _GD_MakeFullPath((DIRFILE*)D, dirfd, name, 0);
+
+  dreturn("\"%s\"", filepath);
+  return filepath;
+}
+
+int _GD_GrabDir(DIRFILE *D, int dirfd, const char *name, int canonical)
+{
+  unsigned int i;
+  char *path, *dir = NULL;
+  void *ptr;
+
+  dtrace("%p, %i, \"%s\", %i", D, dirfd, name, canonical);
+
+  if (canonical)
+    path = _GD_Strdup(D, name);
+  else
+    path = _GD_MakeFullPath(D, dirfd, name, 1);
+
+  if (path == NULL) {
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  dir = dirname(path);
+
+  for (i = 0; i < D->ndir; ++i)
+    if (strcmp(dir, D->dir[i].path) == 0) {
+      D->dir[i].rc++;
+      free(path);
+      dreturn("%i", D->dir[i].fd);
+      return D->dir[i].fd;
+    }
+
+  /* new one */
+  ptr = _GD_Realloc(D, D->dir, sizeof(D->dir[0]) * (D->ndir + 1));
+
+  if (ptr == NULL) {
+    free(path);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  D->dir = ptr;
+  D->dir[D->ndir].rc = 1;
+  D->dir[D->ndir].path = _GD_Strdup(D, dir);
+
+  if (D->dir[D->ndir].path == NULL) {
+    free(path);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+#ifdef GD_NO_DIR_OPEN
+  D->dir[D->ndir].fd = D->ndir;
+  free(path);
+#else
+  if (_GD_AbsPath(name)) {
+    D->dir[D->ndir].fd = open(dir, O_RDONLY);
+  } else {
+    free(path);
+    path = _GD_Strdup(D, name);
+    if (path == NULL) {
+      free(D->dir[D->ndir].path);
+      dreturn("%i", -1);
+      return -1;
+    }
+    D->dir[D->ndir].fd = gd_OpenAt(D, dirfd, dirname(path), O_RDONLY, 0666);
+  }
+  free(path);
+
+  if (D->dir[D->ndir].fd == -1) {
+    free(D->dir[D->ndir].path);
+    dreturn("%i", -1);
+    return -1;
+  }
+#endif
+  D->ndir++;
+
+  dreturn("%i", D->dir[D->ndir - 1].fd);
+  return D->dir[D->ndir - 1].fd;
+}
+
+void _GD_ReleaseDir(DIRFILE *D, int dirfd)
+{
+#ifndef GD_NO_DIR_OPEN
+  unsigned int i;
+#endif
+
+  dtrace("%p, %i", D, dirfd);
+
+#ifdef GD_NO_DIR_OPEN
+  if (--D->dir[dirfd].rc == 0) {
+    free(D->dir[dirfd].path);
+    D->dir[dirfd] = D->dir[--D->ndir];
+  }
+#else
+  for (i = 0; i < D->ndir; ++i)
+    if (D->dir[i].fd == dirfd) {
+      if (--D->dir[i].rc == 0) {
+        free(D->dir[i].path);
+        close(D->dir[i].fd);
+        D->dir[i] = D->dir[--D->ndir];
+      }
+      break;
+    }
+#endif
+
+  dreturnvoid();
+}
+
+/* allocation boilerplates */
+void *_GD_Malloc(DIRFILE *D, size_t size)
+{
+  void *ptr = malloc(size);
+  if (ptr == NULL)
+    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+  return ptr;
+}
+
+void *_GD_Realloc(DIRFILE *restrict D, void *restrict old, size_t size)
+{
+  void *ptr = realloc(old, size);
+  if (ptr == NULL)
+    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+  return ptr;
+}
+
+char *_GD_Strdup(DIRFILE *D, const char *s)
+{
+  char *ptr = strdup(s);
+  if (ptr == NULL)
+    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+  return ptr;
+}
+
 /* vim: ts=2 sw=2 et tw=80
 */
