@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2010 D. V. Wiebe
+/* Copyright (C) 2008-2015 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -20,16 +20,11 @@
  */
 #include "internal.h"
 
-#ifdef STDC_HEADERS
-#include <string.h>
-#include <stdlib.h>
-#endif
-
-void _GD_FreeE(gd_entry_t* entry, int priv)
+void _GD_FreeE(DIRFILE *restrict D, gd_entry_t *restrict entry, int priv)
 {
   int i;
 
-  dtrace("%p, %i", entry, priv);
+  dtrace("%p, %p, %i", D, entry, priv);
 
   if (!entry || entry->field_type == GD_NO_ENTRY) {
     dreturnvoid();
@@ -50,13 +45,12 @@ void _GD_FreeE(gd_entry_t* entry, int priv)
       free(entry->in_fields[0]);
       free(entry->EN(linterp,table));
       if (priv) {
-        free(entry->e->u.linterp.table_path);
+        if (entry->e->u.linterp.table_dirfd > 0)
+          _GD_ReleaseDir(D, entry->e->u.linterp.table_dirfd);
+        free(entry->e->u.linterp.table_file);
         free(entry->e->u.linterp.lut);
-      }
-      break;
-    case GD_RECIP_ENTRY:
-      free(entry->in_fields[0]);
-      free(entry->scalar[0]);
+      } else
+        entry->EN(linterp,table) = NULL;
       break;
     case GD_DIVIDE_ENTRY:
     case GD_MULTIPLY_ENTRY:
@@ -65,9 +59,12 @@ void _GD_FreeE(gd_entry_t* entry, int priv)
       break;
     case GD_BIT_ENTRY:
     case GD_SBIT_ENTRY:
+      free(entry->scalar[0]);
       free(entry->scalar[1]);
-      /* fallthrough */
+      free(entry->in_fields[0]);
+      break;
     case GD_PHASE_ENTRY:
+    case GD_RECIP_ENTRY:
       free(entry->scalar[0]);
       free(entry->in_fields[0]);
       break;
@@ -95,26 +92,46 @@ void _GD_FreeE(gd_entry_t* entry, int priv)
         free(entry->e->u.raw.file[1].name);
       }
       break;
+    case GD_WINDOW_ENTRY:
+      free(entry->scalar[0]);
+      free(entry->in_fields[0]);
+      free(entry->in_fields[1]);
+      break;
+    case GD_MPLEX_ENTRY:
+      free(entry->scalar[0]);
+      free(entry->scalar[1]);
+      free(entry->in_fields[0]);
+      free(entry->in_fields[1]);
+      break;
+    case GD_ALIAS_ENTRY:
+      free(entry->in_fields[0]);
+      break;
     case GD_INDEX_ENTRY:
     case GD_NO_ENTRY:
       break;
   }
 
   if (priv) {
-    free(entry->e->field_list);
-    free(entry->e->vector_list);
+    free(entry->e->alias_list);
+    for (i = 0; i < GD_N_ENTRY_LISTS; ++i)
+      free(entry->e->entry_list[i]);
     free(entry->e->string_value_list);
-    for (i = 0; i < GD_N_ENTYPES; ++i)
-      free(entry->e->type_list[i]);
     free(entry->e->const_value_list);
-    if (entry->e->carray_value_list)
+
+    if (entry->e->carray_value_list) {
       for (i = 0; entry->e->carray_value_list[i].n != 0; ++i)
         free(entry->e->carray_value_list[i].d);
-    free(entry->e->carray_value_list);
+      free(entry->e->carray_value_list);
+    }
+
     if (entry->e->n_meta > -1)
       free(entry->e->p.meta_entry);
     free(entry->e);
     free(entry);
+  } else {
+    entry->field = NULL;
+    memset(entry->in_fields, 0, sizeof(char*) * GD_MAX_LINCOM);
+    memset(entry->scalar, 0, sizeof(char*) * GD_MAX_LINCOM * 2);
   }
 
   dreturnvoid();
@@ -125,51 +142,56 @@ gd_entry_t* gd_free_entry_strings(gd_entry_t* entry) gd_nothrow
 {
   dtrace("%p", entry);
 
-  _GD_FreeE(entry, 0);
+  _GD_FreeE(NULL, entry, 0);
 
   dreturn("%p", entry);
   return entry;
 }
 
-static void _GD_GetScalar(DIRFILE* D, gd_entry_t* E, int i, gd_type_t type,
-    void* data)
+static int _GD_GetScalar(DIRFILE *restrict D, gd_entry_t *restrict E, int i,
+    gd_type_t type, void *restrict data, int err)
 {
   void *ptr = NULL;
-  gd_entry_t* C;
-  int repr;
+  gd_entry_t* C = NULL;
+  int repr, e = 0;
   char* field_code;
   const char* scalar = E->scalar[i];
   int index = E->scalar_ind[i];
 
-  dtrace("%p, %p, %i, %i, %p", D, E, i, type, data);
+  dtrace("%p, %p, %i, %i, %p, %i", D, E, i, type, data, err);
 
   if (scalar != NULL) {
-    C = _GD_FindFieldAndRepr(D, scalar, &field_code, &repr, NULL, 0);
+    C = _GD_FindFieldAndRepr(D, scalar, &field_code, &repr, NULL, 0, 1);
 
     if (D->error) {
-      dreturnvoid();
-      return;
+      dreturn("%i", 1);
+      return 1;
     }
 
-    if (C == NULL)
-      _GD_SetError(D, GD_E_BAD_SCALAR, GD_E_SCALAR_CODE, E->field, 0,
-          field_code);
-    else if (C->field_type != GD_CONST_ENTRY &&
-        C->field_type != GD_CARRAY_ENTRY) 
-      _GD_SetError(D, GD_E_BAD_SCALAR, GD_E_SCALAR_TYPE, E->field, 0,
-          field_code);
-    else {
+    if (C == NULL) {
+      if (err)
+        _GD_SetError(D, GD_E_BAD_SCALAR, GD_E_SCALAR_CODE, E->field, 0,
+            field_code);
+      e = 1;
+    } else if (C->field_type != GD_CONST_ENTRY &&
+        C->field_type != GD_CARRAY_ENTRY)
+    {
+      if (err)
+        _GD_SetError(D, GD_E_BAD_SCALAR, GD_E_SCALAR_TYPE, E->field, 0,
+            field_code);
+      e = 1;
+    } else {
       if (C->field_type == GD_CONST_ENTRY) {
         index = 0;
         E->scalar_ind[i] = -1;
+      } else if (index < 0) {
+        /* a CARRAY masquerading as a CONST; default to the first element */
+        index = E->scalar_ind[i] = 0;
       }
 
-      if ((D->flags & GD_ACCMODE) == GD_RDWR) {
-        ptr = realloc(C->e->u.scalar.client, (C->e->u.scalar.n_client + 1) *
-            sizeof(gd_entry_t*));
-        if (ptr == NULL)
-          _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-      }
+      if ((D->flags & GD_ACCMODE) == GD_RDWR)
+        ptr = _GD_Realloc(D, C->e->u.scalar.client,
+            (C->e->u.scalar.n_client + 1) * sizeof(gd_entry_t*));
 
       _GD_DoField(D, C, repr, index, 1, type, data);
 
@@ -183,65 +205,99 @@ static void _GD_GetScalar(DIRFILE* D, gd_entry_t* E, int i, gd_type_t type,
       free(field_code);
   }
 
-  dreturnvoid();
+  dreturn("%i", e);
+  return e;
 }
 
 /* resolve non-literal scalars */
-int _GD_CalculateEntry(DIRFILE* D, gd_entry_t* E)
+int _GD_CalculateEntry(DIRFILE *restrict D, gd_entry_t *restrict E, int err)
 {
-  int i;
+  int i, e = 0, cs = 0;
 
-  dtrace("%p, %p", D, E);
+  dtrace("%p, %p, %i", D, E, err);
 
   switch(E->field_type) {
     case GD_RAW_ENTRY:
-      _GD_GetScalar(D, E, 0, GD_UINT16, &E->EN(raw,spf));
+      e = _GD_GetScalar(D, E, 0, GD_UINT_TYPE, &E->EN(raw,spf), err);
       break;
     case GD_POLYNOM_ENTRY:
-      E->comp_scal = 0;
       for (i = 0; i <= E->EN(polynom,poly_ord); ++i) {
-        _GD_GetScalar(D, E, i, GD_COMPLEX128, &E->EN(polynom,ca)[i]);
-        E->EN(polynom,a)[i] = creal(E->EN(polynom,ca)[i]);
+        if (_GD_GetScalar(D, E, i, GD_COMPLEX128, &E->EN(polynom,ca)[i], err))
+          e = 1;
+        else {
+          E->EN(polynom,a)[i] = creal(E->EN(polynom,ca)[i]);
 
-        if (cimag(E->EN(polynom,ca)[i]))
-          E->comp_scal = 1;
+          if (!cs && cimag(E->EN(polynom,ca)[i]))
+            cs = 1;
+        }
 
         if (D->error)
           break;
       }
       break;
     case GD_LINCOM_ENTRY:
-      E->comp_scal = 0;
       for (i = 0; i < E->EN(lincom,n_fields); ++i) {
-        _GD_GetScalar(D, E, i, GD_COMPLEX128, &E->EN(lincom,cm)[i]);
-        E->EN(lincom,m)[i] = creal(E->EN(lincom,cm)[i]);
+        if (_GD_GetScalar(D, E, i, GD_COMPLEX128, &E->EN(lincom,cm)[i], err))
+          e = 1;
+        else {
+          E->EN(lincom,m)[i] = creal(E->EN(lincom,cm)[i]);
 
-        if (cimag(E->EN(lincom,cm)[i]))
-          E->comp_scal = 1;
+          if (!cs && cimag(E->EN(lincom,cm)[i]))
+            cs = 1;
+        }
 
-        _GD_GetScalar(D, E, i + GD_MAX_LINCOM, GD_COMPLEX128,
-            &E->EN(lincom,cb)[i]);
-        E->EN(lincom,b)[i] = creal(E->EN(lincom,cb)[i]);
+        if (_GD_GetScalar(D, E, i + GD_MAX_LINCOM, GD_COMPLEX128,
+            &E->EN(lincom,cb)[i], err))
+        {
+          e = 1;
+        } else {
+          E->EN(lincom,b)[i] = creal(E->EN(lincom,cb)[i]);
 
-        if (cimag(E->EN(lincom,cb)[i]))
-          E->comp_scal = 1;
+          if (!cs && cimag(E->EN(lincom,cb)[i]))
+            cs = 1;
+        }
 
         if (D->error)
           break;
       }
       break;
     case GD_RECIP_ENTRY:
-      _GD_GetScalar(D, E, 0, GD_COMPLEX128, &E->EN(recip,cdividend));
-      E->EN(recip,dividend) = creal(E->EN(recip,cdividend));
-      E->comp_scal = (cimag(E->EN(recip,cdividend)) == 0) ? 0 : 1;
+      if (_GD_GetScalar(D, E, 0, GD_COMPLEX128, &E->EN(recip,cdividend), err))
+        e = 1;
+      else {
+        E->EN(recip,dividend) = creal(E->EN(recip,cdividend));
+        if (cimag(E->EN(recip,cdividend)))
+          cs = 1;
+      }
       break;
     case GD_BIT_ENTRY:
     case GD_SBIT_ENTRY:
-      _GD_GetScalar(D, E, 0, GD_INT16, &E->EN(bit,bitnum));
-      _GD_GetScalar(D, E, 1, GD_INT16, &E->EN(bit,numbits));
+      e = _GD_GetScalar(D, E, 0, GD_INT_TYPE, &E->EN(bit,bitnum), err);
+      e |= _GD_GetScalar(D, E, 1, GD_INT_TYPE, &E->EN(bit,numbits), err);
       break;
     case GD_PHASE_ENTRY:
-      _GD_GetScalar(D, E, 0, GD_INT64, &E->EN(phase,shift));
+      e = _GD_GetScalar(D, E, 0, GD_INT64, &E->EN(phase,shift), err);
+      break;
+    case GD_WINDOW_ENTRY:
+      switch (E->EN(window,windop)) {
+        case GD_WINDOP_EQ:
+        case GD_WINDOP_NE:
+          e = _GD_GetScalar(D, E, 0, GD_INT64, &E->EN(window,threshold.i), err);
+          break;
+        case GD_WINDOP_SET:
+        case GD_WINDOP_CLR:
+          e = _GD_GetScalar(D, E, 0, GD_UINT64, &E->EN(window,threshold.u),
+              err);
+          break;
+        default:
+          e = _GD_GetScalar(D, E, 0, GD_FLOAT64, &E->EN(window,threshold.r),
+              err);
+          break;
+      }
+      break;
+    case GD_MPLEX_ENTRY:
+      e = _GD_GetScalar(D, E, 0, GD_INT_TYPE, &E->EN(mplex,count_val), err);
+      e |= _GD_GetScalar(D, E, 1, GD_INT_TYPE, &E->EN(mplex,period), err);
       break;
     case GD_NO_ENTRY:
     case GD_LINTERP_ENTRY:
@@ -251,23 +307,28 @@ int _GD_CalculateEntry(DIRFILE* D, gd_entry_t* E)
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
     case GD_INDEX_ENTRY:
+    case GD_ALIAS_ENTRY:
       break;
   }
 
-  if (!D->error)
-    E->e->calculated = 1;
+  if (!e)
+    E->flags |= GD_EN_CALC;
 
-  dreturn("%i", E->e->calculated);
-  return E->e->calculated;
+  if (cs)
+    E->flags |= GD_EN_COMPSCAL;
+  else
+    E->flags &= ~GD_EN_COMPSCAL;
+
+  dreturn("%i", !e);
+  return !e;
 }
 
-const char* gd_raw_filename(DIRFILE* D, const char* field_code_in) gd_nothrow
+char* gd_raw_filename(DIRFILE* D, const char* field_code) gd_nothrow
 {
-  int repr;
-  char* field_code;
+  char *filename;
   gd_entry_t *E;
 
-  dtrace("%p, \"%s\"", D, field_code_in);
+  dtrace("%p, \"%s\"", D, field_code);
 
   _GD_ClearError(D);
 
@@ -278,7 +339,7 @@ const char* gd_raw_filename(DIRFILE* D, const char* field_code_in) gd_nothrow
   }
 
   /* Check field */
-  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1);
+  E = _GD_FindEntry(D, field_code, NULL, 1, 1);
 
   if (D->error) {
     dreturn("%p", NULL);
@@ -291,39 +352,39 @@ const char* gd_raw_filename(DIRFILE* D, const char* field_code_in) gd_nothrow
     return NULL;
   }
 
-  if (field_code != field_code_in)
-    free(field_code);
-
   if (E->e->u.raw.file[0].name == NULL) {
     /* ensure encoding sybtype is known */
-    if (!_GD_Supports(D, E, 0)) {
+    if (!_GD_Supports(D, E, GD_EF_NAME)) {
       dreturn("%p", NULL);
       return NULL;
     }
 
-    if (E->e->u.raw.file[0].encoding == GD_ENC_UNKNOWN) {
-      _GD_SetError(D, GD_E_UNKNOWN_ENCODING, 0, NULL, 0, NULL);
+    if (E->e->u.raw.file[0].subenc == GD_ENC_UNKNOWN) {
+      _GD_SetError(D, GD_E_UNKNOWN_ENCODING, GD_E_UNENC_UNDET, NULL, 0, NULL);
       dreturn("%p", NULL);
       return NULL;
-    } else if (_GD_SetEncodedName(D, E->e->u.raw.file, E->e->u.raw.filebase, 0))
+    } else if ((*_GD_ef[E->e->u.raw.file[0].subenc].name)(D,
+          (const char*)D->fragment[E->fragment_index].enc_data,
+          E->e->u.raw.file, E->e->u.raw.filebase, 0, 0))
     {
       dreturn("%p", NULL);
       return NULL;
     }
   }
 
-  dreturn("%p", E->e->u.raw.file[0].name);
-  return E->e->u.raw.file[0].name;
+  filename = _GD_MakeFullPath(D, D->fragment[E->fragment_index].dirfd,
+      E->e->u.raw.file->name, 1);
+
+  dreturn("%p", filename);
+  return filename;
 }
 
-int gd_entry(DIRFILE* D, const char* field_code_in, gd_entry_t* entry)
-  gd_nothrow
+int gd_entry(DIRFILE* D, const char* field_code, gd_entry_t* entry) gd_nothrow
 {
-  int i, repr;
+  int i;
   gd_entry_t *E;
-  char* field_code;
 
-  dtrace("%p, \"%s\", %p", D, field_code_in, entry);
+  dtrace("%p, \"%s\", %p", D, field_code, entry);
 
   if (D->flags & GD_INVALID) {
     _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
@@ -333,25 +394,17 @@ int gd_entry(DIRFILE* D, const char* field_code_in, gd_entry_t* entry)
 
   _GD_ClearError(D);
 
-  /* get rid of the represenation, if any */
-  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1);
+  E = _GD_FindEntry(D, field_code, NULL, 1, 1);
 
   if (D->error) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
     dreturn("%i", -1);
     return -1;
   }
 
-  if (field_code != field_code_in)
-    free(field_code);
-
   /* Calculate the entry, if necessary */
-  if (!E->e->calculated)
-    _GD_CalculateEntry(D, E);
-
-  if (D->error) {
-    dreturn("%i", -1);
-    return -1;
-  }
+  if (!(E->flags & GD_EN_CALC))
+    _GD_CalculateEntry(D, E, 0);
 
   /* now copy to the user supplied buffer */
   memcpy(entry, E, sizeof(gd_entry_t));
@@ -404,11 +457,26 @@ int gd_entry(DIRFILE* D, const char* field_code_in, gd_entry_t* entry)
       if (E->scalar[0])
         entry->scalar[0] = strdup(E->scalar[0]);
       break;
+    case GD_WINDOW_ENTRY:
+      entry->in_fields[0] = strdup(E->in_fields[0]);
+      entry->in_fields[1] = strdup(E->in_fields[1]);
+      if (E->scalar[0])
+        entry->scalar[0] = strdup(E->scalar[0]);
+      break;
+    case GD_MPLEX_ENTRY:
+      entry->in_fields[0] = strdup(E->in_fields[0]);
+      entry->in_fields[1] = strdup(E->in_fields[1]);
+      if (E->scalar[0])
+        entry->scalar[0] = strdup(E->scalar[0]);
+      if (E->scalar[1])
+        entry->scalar[1] = strdup(E->scalar[1]);
+      break;
     case GD_INDEX_ENTRY:
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
     case GD_STRING_ENTRY:
     case GD_NO_ENTRY:
+    case GD_ALIAS_ENTRY:
       break;
   }
 
@@ -416,13 +484,126 @@ int gd_entry(DIRFILE* D, const char* field_code_in, gd_entry_t* entry)
   return 0;
 }
 
-gd_entype_t gd_entry_type(DIRFILE* D, const char* field_code_in) gd_nothrow
+const char *gd_alias_target(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
+
+  if (E == NULL) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  if (E->field_type != GD_ALIAS_ENTRY) {
+    _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_BAD, NULL, 0, field_code);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  dreturn("\"%s\"", E->in_fields[0]);
+  return E->in_fields[0];
+}
+
+const char **gd_aliases(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+  int j = 1;
+  unsigned u, n;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  n = gd_naliases(D, field_code) + 1;
+
+  if (D->error) {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
+
+  if (E == NULL) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  E->e->alias_list = (const char**)_GD_Realloc(D, E->e->alias_list,
+      sizeof(const char *) * n);
+
+  if (D->error) {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  E->e->alias_list[0] = E->field;
+
+  for (u = 0; u < D->n_entries; ++u)
+    if (D->entry[u]->field_type == GD_ALIAS_ENTRY &&
+        D->entry[u]->e->entry[0] == E)
+    {
+      E->e->alias_list[j++] = D->entry[u]->field;
+    }
+
+  /* terminate */
+  E->e->alias_list[j] = NULL;
+
+  dreturn("%p", E->e->alias_list);
+  return E->e->alias_list;
+}
+
+unsigned int gd_naliases(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+  unsigned u, n = 1;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%u", 0);
+    return 0;
+  }
+
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
+
+  if (E == NULL) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    dreturn("%u", 0);
+    return 0;
+  }
+
+  for (u = 0; u < D->n_entries; ++u)
+    if (D->entry[u]->field_type == GD_ALIAS_ENTRY &&
+        D->entry[u]->e->entry[0] == E)
+    {
+      n++;
+    }
+
+  dreturn("%u", n);
+  return n;
+}
+
+gd_entype_t gd_entry_type(DIRFILE* D, const char* field_code) gd_nothrow
 {
   gd_entry_t* E;
-  char* field_code;
-  int repr;
 
-  dtrace("%p, \"%s\"", D, field_code_in);
+  dtrace("%p, \"%s\"", D, field_code);
 
   if (D->flags & GD_INVALID) {
     _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
@@ -432,28 +613,22 @@ gd_entype_t gd_entry_type(DIRFILE* D, const char* field_code_in) gd_nothrow
 
   _GD_ClearError(D);
 
-  /* get rid of the represenation, if any */
-  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1);
+  E = _GD_FindEntry(D, field_code, NULL, 1, 1);
 
   if (D->error) {
     dreturn("%i", GD_NO_ENTRY);
     return GD_NO_ENTRY;
   }
 
-  if (field_code != field_code_in)
-    free(field_code);
-
   dreturn("%i", E->field_type);
   return E->field_type;
 }
 
-int gd_fragment_index(DIRFILE* D, const char* field_code_in) gd_nothrow
+int gd_fragment_index(DIRFILE *D, const char *field_code) gd_nothrow
 {
-  gd_entry_t* E;
-  char* field_code;
-  int repr;
+  gd_entry_t *E;
 
-  dtrace("%p, \"%s\"", D, field_code_in);
+  dtrace("%p, \"%s\"", D, field_code);
 
   if (D->flags & GD_INVALID) {
     _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
@@ -463,21 +638,128 @@ int gd_fragment_index(DIRFILE* D, const char* field_code_in) gd_nothrow
 
   _GD_ClearError(D);
 
-  /* get rid of the represenation, if any */
-  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1);
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
 
-  if (D->error) {
+  if (E == NULL) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
     dreturn("%i", -1);
     return -1;
   }
 
-  if (field_code != field_code_in)
-    free(field_code);
-
   dreturn("%i", E->fragment_index);
   return E->fragment_index;
 }
 
+int gd_hide(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%i", -1);
+    return -1;
+  } else if ((D->flags & GD_ACCMODE) != GD_RDWR) {
+    _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  _GD_ClearError(D);
+
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
+
+  if (D->error) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT) {
+    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
+        D->fragment[E->fragment_index].cname);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  if (!(E->flags & GD_EN_HIDDEN)) {
+    E->flags |= GD_EN_HIDDEN;
+    D->fragment[E->fragment_index].modified = 1;
+  }
+
+  dreturn("%i", 0);
+  return 0;
+}
+
+int gd_hidden(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  _GD_ClearError(D);
+
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
+
+  if (D->error) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  dreturn("%i", (E->flags & GD_EN_HIDDEN) ? 1 : 0);
+  return (E->flags & GD_EN_HIDDEN) ? 1 : 0;
+}
+
+int gd_unhide(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%i", -1);
+    return -1;
+  } else if ((D->flags & GD_ACCMODE) != GD_RDWR) {
+    _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  _GD_ClearError(D);
+
+  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
+
+  if (D->error) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT) {
+    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
+        D->fragment[E->fragment_index].cname);
+    dreturn("%i", -1);
+    return -1;
+  }
+
+  if (E->flags & GD_EN_HIDDEN) {
+    E->flags &= ~GD_EN_HIDDEN;
+    D->fragment[E->fragment_index].modified = 1;
+  }
+
+  dreturn("%i", 0);
+  return 0;
+}
+
 int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
 {
   int i, repr;
@@ -494,8 +776,7 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
 
   _GD_ClearError(D);
 
-  /* get rid of the representation, if any */
-  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1);
+  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1, 1);
 
   if (D->error) {
     dreturn("%i", -1);
@@ -506,18 +787,20 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
     free(field_code);
 
   /* calculate scalars */
-  if (!E->e->calculated)
-    _GD_CalculateEntry(D, E);
+  if (!(E->flags & GD_EN_CALC))
+    _GD_CalculateEntry(D, E, 1);
 
   /* check input fields */
   switch (E->field_type) {
     case GD_LINCOM_ENTRY:
       for (i = 0; i < E->EN(lincom,n_fields); ++i)
-        _GD_BadInput(D, E, i);
+        _GD_BadInput(D, E, i, GD_NO_ENTRY, 1);
       break;
     case GD_DIVIDE_ENTRY:
     case GD_MULTIPLY_ENTRY:
-      _GD_BadInput(D, E, 1);
+    case GD_WINDOW_ENTRY:
+    case GD_MPLEX_ENTRY:
+      _GD_BadInput(D, E, 1, GD_NO_ENTRY, 1);
       /* fallthrough */
     case GD_LINTERP_ENTRY:
     case GD_BIT_ENTRY:
@@ -525,7 +808,7 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
     case GD_POLYNOM_ENTRY:
     case GD_SBIT_ENTRY:
     case GD_RECIP_ENTRY:
-      _GD_BadInput(D, E, 0);
+      _GD_BadInput(D, E, 0, GD_NO_ENTRY, 1);
       /* Fallthrough */
     case GD_RAW_ENTRY:
     case GD_CONST_ENTRY:
@@ -533,6 +816,7 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
     case GD_STRING_ENTRY:
     case GD_INDEX_ENTRY:
     case GD_NO_ENTRY:
+    case GD_ALIAS_ENTRY:
       break;
   }
 
@@ -544,3 +828,45 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
   dreturn("%i", 0);
   return 0;
 }
+
+char *gd_linterp_tablename(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  gd_entry_t *E;
+  char *table;
+
+  dtrace("%p, \"%s\"", D, field_code);
+
+  if (D->flags & GD_INVALID) {
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  _GD_ClearError(D);
+
+  E = _GD_FindEntry(D, field_code, NULL, 1, 1);
+
+  if (D->error) {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  if (E->field_type != GD_LINTERP_ENTRY) {
+    _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_BAD, NULL, 0, field_code);
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  /* initialise */
+  if (E->e->u.linterp.table_file == NULL)
+    if (_GD_SetTablePath(D, E, E->e)) {
+      dreturn("%p", NULL);
+      return NULL;
+    }
+
+  table = _GD_MakeFullPath(D, E->e->u.linterp.table_dirfd,
+      E->e->u.linterp.table_file, 1);
+
+  dreturn("%s", table);
+  return table;
+}
