@@ -1,6 +1,6 @@
 /* Copyright (C) 2003-2005 C. Barth Netterfield
  * Copyright (C) 2003-2005 Theodore Kisner
- * Copyright (C) 2005-2010 D. V. Wiebe
+ * Copyright (C) 2005-2015 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -22,18 +22,10 @@
  */
 #include "internal.h"
 
-#ifdef STDC_HEADERS
-#include <inttypes.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdlib.h>
-#endif
-
-static size_t _GD_DoRawOut(DIRFILE *D, gd_entry_t *E, off64_t s0,
-    size_t ns, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoRawOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t s0, size_t ns, gd_type_t data_type, const void *restrict data_in)
 {
-  size_t n_wrote;
+  ssize_t n_wrote;
   void *databuffer;
 
   /* check protection */
@@ -44,7 +36,8 @@ static size_t _GD_DoRawOut(DIRFILE *D, gd_entry_t *E, off64_t s0,
     return 0;
   }
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, s0, ns, data_type, data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)s0, ns,
+      data_type, data_in);
 
   if (s0 < D->fragment[E->fragment_index].frame_offset * E->EN(raw,spf)) {
     _GD_SetError(D, GD_E_RANGE, GD_E_OUT_OF_RANGE, NULL, 0, NULL);
@@ -74,79 +67,61 @@ static size_t _GD_DoRawOut(DIRFILE *D, gd_entry_t *E, off64_t s0,
     return 0;
   }
 
-  if (_gd_ef[E->e->u.raw.file[0].encoding].ecor) {
-    /* convert to/from middle-ended doubles */
-    if ((E->EN(raw,data_type) == GD_FLOAT64 || E->EN(raw,data_type) ==
-          GD_COMPLEX128) &&
-        D->fragment[E->fragment_index].byte_sex & GD_ARM_FLAG)
-    {
-      _GD_ArmEndianise((uint64_t*)databuffer, E->EN(raw,data_type) & GD_COMPLEX,
-          ns);
-    }
+  /* fix endianness, if necessary */
+  if (_GD_ef[E->e->u.raw.file[0].subenc].flags & GD_EF_ECOR)
+    _GD_FixEndianness(databuffer, ns, E->EN(raw,data_type), 0,
+        D->fragment[E->fragment_index].byte_sex);
 
-    if (D->fragment[E->fragment_index].byte_sex &
-#ifdef WORDS_BIGENDIAN
-           GD_LITTLE_ENDIAN
-#else
-           GD_BIG_ENDIAN
-#endif
-           )
-    {
-      if (E->EN(raw,data_type) & GD_COMPLEX)
-        _GD_FixEndianness((char *)databuffer, E->e->u.raw.size / 2, ns * 2);
-      else
-        _GD_FixEndianness((char *)databuffer, E->e->u.raw.size, ns);
-    }
-  }
   /* write data to file. */
-
-  if (E->e->u.raw.file[0].fp < 0) {
-    /* open file for reading / writing if not already opened */
-
-    if (_GD_SetEncodedName(D, E->e->u.raw.file, E->e->u.raw.filebase, 0)) {
-      free(databuffer);
-      dreturn("%i", 0);
-      return 0;
-    } else if ((*_gd_ef[E->e->u.raw.file[0].encoding].open)(E->e->u.raw.file,
-          D->flags & GD_ACCMODE, 1))
-    {
-      _GD_SetError(D, GD_E_RAW_IO, 0, E->e->u.raw.file[0].name, errno, NULL);
-      free(databuffer);
-      dreturn("%i", 0);
-      return 0;
-    }
-  }
-
-  if ((*_gd_ef[E->e->u.raw.file[0].encoding].seek)(E->e->u.raw.file, s0,
-        E->EN(raw,data_type), 1) == -1)
+  if (_GD_InitRawIO(D, E, NULL, -1, NULL, 0, GD_FILE_WRITE,
+        _GD_FileSwapBytes(D, E)))
   {
-      _GD_SetError(D, GD_E_RAW_IO, 0, E->e->u.raw.file[0].name, errno, NULL);
-      free(databuffer);
-      dreturn("%i", 0);
-      return 0;
+    free(databuffer);
+    dreturn("%i", 0);
+    return 0;
   }
 
-  n_wrote = (*_gd_ef[E->e->u.raw.file[0].encoding].write)(E->e->u.raw.file,
-      databuffer, E->EN(raw,data_type), ns);
+  if (_GD_DoSeek(D, E, _GD_ef + E->e->u.raw.file[0].subenc, s0, GD_FILE_WRITE)
+      == -1)
+  {
+    free(databuffer);
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  n_wrote = _GD_WriteOut(E, _GD_ef + E->e->u.raw.file[0].subenc, databuffer,
+      E->EN(raw,data_type), ns, 0);
+
+  if (n_wrote < 0) {
+    _GD_SetEncIOError(D, GD_E_IO_WRITE, E->e->u.raw.file + 0);
+    n_wrote = 0;
+  }
 
   free(databuffer);
 
-  dreturn("%zu", n_wrote);
-  return n_wrote;
+  dreturn("%" PRNssize_t, n_wrote);
+  return (size_t)n_wrote;
 }
 
-static size_t _GD_DoLinterpOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoLinterpOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   size_t n_wrote;
   int dir = -1, i;
   double *tmpbuf;
-  struct _gd_lut *tmp_lut;
+  struct gd_lut_ *tmp_lut;
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first_samp, num_samp, data_type,
-      data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0)) {
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+  
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
     return 0;
   }
@@ -189,7 +164,7 @@ static size_t _GD_DoLinterpOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  tmpbuf = (double *)_GD_Alloc(D, GD_FLOAT64, num_samp);
+  tmpbuf = _GD_Alloc(D, GD_FLOAT64, num_samp);
   if (tmpbuf == NULL) {
     free(tmpbuf);
     dreturn("%i", 0);
@@ -205,11 +180,9 @@ static size_t _GD_DoLinterpOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
   }
 
   /* Make the reverse lut */
-  tmp_lut = (struct _gd_lut *)malloc(E->e->u.linterp.table_len *
-      sizeof(struct _gd_lut));
+  tmp_lut = _GD_Malloc(D, E->e->u.linterp.table_len * sizeof(*tmp_lut));
   if (tmp_lut == NULL) {
     free(tmpbuf);
-    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
     dreturn("%i", 0);
     return 0;
   }
@@ -229,23 +202,24 @@ static size_t _GD_DoLinterpOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], E->e->repr[0], first_samp,
-      num_samp, GD_FLOAT64, tmpbuf);
+  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp, num_samp, GD_FLOAT64,
+      tmpbuf);
 
   free(tmpbuf);
 
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
   return n_wrote;
 }
 
-static size_t _GD_DoLincomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoLincomOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   size_t n_wrote;
   void* tmpbuf;
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first_samp, num_samp, data_type,
-      data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
   /* we cannot write to LINCOM fields that are a linear combination */
   /* of more than one raw field (no way to know how to split data). */
@@ -256,7 +230,13 @@ static size_t _GD_DoLincomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  if (_GD_BadInput(D, E, 0)) {
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
     return 0;
   }
@@ -271,7 +251,7 @@ static size_t _GD_DoLincomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
 
   memcpy(tmpbuf, data_in, num_samp * GD_SIZE(data_type));
 
-  if (E->comp_scal) {
+  if (E->flags & GD_EN_COMPSCAL) {
 #ifdef GD_NO_C99_API
     double cm[1][2];
     double cb[1][2];
@@ -305,16 +285,17 @@ static size_t _GD_DoLincomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], E->e->repr[0], first_samp,
-      num_samp, data_type, tmpbuf);
+  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp, num_samp, data_type,
+      tmpbuf);
   free(tmpbuf);
 
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
   return n_wrote;
 }
 
-static size_t _GD_DoBitOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoBitOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   uint64_t *tmpbuf;
   uint64_t *readbuf;
@@ -322,16 +303,22 @@ static size_t _GD_DoBitOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
   const uint64_t mask = (E->EN(bit,numbits) == 64) ? 0xffffffffffffffffULL :
     ((uint64_t)1 << E->EN(bit,numbits)) - 1;
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first_samp, num_samp, data_type,
-      data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0)) {
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
     dreturn("%i", 0);
     return 0;
   }
 
-  tmpbuf = (uint64_t *)_GD_Alloc(D, GD_UINT64, num_samp);
-  readbuf = (uint64_t *)_GD_Alloc(D, GD_UINT64, num_samp);
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  tmpbuf = _GD_Alloc(D, GD_UINT64, num_samp);
+  readbuf = _GD_Alloc(D, GD_UINT64, num_samp);
 
   if (tmpbuf == NULL || readbuf == NULL) {
     free(tmpbuf);
@@ -361,47 +348,61 @@ static size_t _GD_DoBitOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
       (tmpbuf[i] & mask) << E->EN(bit,bitnum);
 
   /* write the modified data out */
-  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], E->e->repr[0], first_samp,
-      num_samp, GD_UINT64, (void*)readbuf);
+  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp, num_samp, GD_UINT64,
+      (void*)readbuf);
 
   free(readbuf);
   free(tmpbuf);
 
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
   return n_wrote;
 }
 
-static size_t _GD_DoPhaseOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoPhaseOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   size_t n_wrote;
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first_samp, num_samp, data_type,
-      data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0)) {
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
     dreturn("%i", 0);
     return 0;
   }
 
-  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], E->e->repr[0], first_samp +
-      E->EN(phase,shift), num_samp, data_type, data_in);
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
+    dreturn("%i", 0);
+    return 0;
+  }
 
-  dreturn("%zu", n_wrote);
+  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp + E->EN(phase,shift),
+      num_samp, data_type, data_in);
+
+  dreturn("%" PRNsize_t, n_wrote);
 
   return n_wrote;
 }
 
-static size_t _GD_DoRecipOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoRecipOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   size_t n_wrote;
   void* tmpbuf;
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first_samp, num_samp, data_type,
-      data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0)) {
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
     return 0;
   }
@@ -417,7 +418,7 @@ static size_t _GD_DoRecipOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
   memcpy(tmpbuf, data_in, num_samp * GD_SIZE(data_type));
 
   /* calculate x = a/y instead of y = a/x */
-  if (E->comp_scal)
+  if (E->flags & GD_EN_COMPSCAL)
     _GD_CInvertData(D, tmpbuf, data_type, E->EN(recip,cdividend), num_samp);
   else
     _GD_InvertData(D, tmpbuf, data_type, E->EN(recip,dividend), num_samp);
@@ -428,23 +429,24 @@ static size_t _GD_DoRecipOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], E->e->repr[0], first_samp,
-      num_samp, data_type, tmpbuf);
+  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp, num_samp, data_type,
+      tmpbuf);
   free(tmpbuf);
 
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
 
   return n_wrote;
 }
 
-static size_t _GD_DoPolynomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+static size_t _GD_DoPolynomOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   size_t n_wrote;
   void* tmpbuf;
 
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first_samp, num_samp, data_type,
-      data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
   /* we cannot write to POLYNOM fields that are quadradic or higher order */
 
@@ -454,7 +456,13 @@ static size_t _GD_DoPolynomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  if (_GD_BadInput(D, E, 0)) {
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
     return 0;
   }
@@ -475,7 +483,7 @@ static size_t _GD_DoPolynomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
 
   memcpy(tmpbuf, data_in, num_samp * GD_SIZE(data_type));
 
-  if (E->comp_scal) {
+  if (E->flags & GD_EN_COMPSCAL) {
 #ifdef GD_NO_C99_API
     double cm[1][2];
     double cb[1][2];
@@ -509,18 +517,146 @@ static size_t _GD_DoPolynomOut(DIRFILE* D, gd_entry_t *E, off64_t first_samp,
     return 0;
   }
 
-  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], E->e->repr[0], first_samp,
-      num_samp, data_type, tmpbuf);
+  n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp, num_samp, data_type,
+      tmpbuf);
   free(tmpbuf);
 
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
   return n_wrote;
 }
 
-static size_t _GD_DoConstOut(DIRFILE* D, gd_entry_t *E, off64_t first,
-    size_t len, gd_type_t data_type, const void *data_in)
+#define MPLEX(t) \
+  for (i = 0; i < n; i++) \
+    if (B[i] == val) \
+      ((t*)A)[i] = ((t*)C)[i * spfB / spfA];
+
+#define MPLEXC(t) \
+  do { \
+    for (i = 0; i < n; i++) \
+      if (B[i] == val) { \
+        ((t*)A)[i * 2] = ((t*)C)[i * 2 * spfB / spfA]; \
+        ((t*)A)[i * 2 + 1] = ((t*)C)[i * 2 * spfB / spfA + 1]; \
+      } \
+  } while (0)
+
+static void _GD_MplexOutData(DIRFILE *restrict D, void *restrict A,
+    unsigned int spfA, const int *restrict B, unsigned int spfB,
+    const void *restrict C, gd_type_t type, int val, size_t n)
 {
-  dtrace("%p, %p, %lli, %zu, 0x%x, %p", D, E, first, len, data_type, data_in);
+  size_t i;
+
+  dtrace("%p, %p, %u, %p, %u, %p, 0x%X %i, %" PRNsize_t, D, A, spfA, B, spfB, C,
+      type, val, n);
+
+  switch (type) {
+    case GD_NULL:                        break;
+    case GD_UINT8:      MPLEX( uint8_t); break;
+    case GD_INT8:       MPLEX(  int8_t); break;
+    case GD_UINT16:     MPLEX(uint16_t); break;
+    case GD_INT16:      MPLEX( int16_t); break;
+    case GD_UINT32:     MPLEX(uint32_t); break;
+    case GD_INT32:      MPLEX( int32_t); break;
+    case GD_UINT64:     MPLEX(uint64_t); break;
+    case GD_INT64:      MPLEX( int64_t); break;
+    case GD_FLOAT32:    MPLEX(   float); break;
+    case GD_FLOAT64:    MPLEX(  double); break;
+    case GD_COMPLEX64:  MPLEXC(  float); break;
+    case GD_COMPLEX128: MPLEXC( double); break;
+    default:            _GD_SetError(D, GD_E_BAD_TYPE, type, NULL, 0, NULL);
+                        break;
+  }
+
+  dreturnvoid();
+}
+
+static size_t _GD_DoMplexOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
+{
+  size_t n_wrote = 0, num_samp2;
+  void *tmpbuf;
+  int *cntbuf;
+  off64_t first_samp2;
+  unsigned int spf1, spf2;
+
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
+
+  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  if (_GD_BadInput(D, E, 1, GD_NO_ENTRY, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* read the data to be modified */
+  spf1 = _GD_GetSPF(D, E->e->entry[0]);
+  spf2 = _GD_GetSPF(D, E->e->entry[1]);
+
+  if (D->error) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  num_samp2 = (int)ceil((double)num_samp * spf2 / spf1);
+  first_samp2 = first_samp * spf2 / spf1;
+
+  tmpbuf = _GD_Alloc(D, data_type, num_samp);
+  cntbuf = _GD_Alloc(D, GD_INT_TYPE, num_samp2);
+
+  if (tmpbuf == NULL || cntbuf == NULL) {
+    free(cntbuf);
+    free(tmpbuf);
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  memset(tmpbuf, 0, num_samp * GD_SIZE(data_type));
+  memset(cntbuf, 0, num_samp2 * GD_SIZE(GD_INT_TYPE));
+
+  _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp, num_samp, data_type,
+      tmpbuf);
+
+  _GD_DoField(D, E->e->entry[1], E->e->repr[1], first_samp2, num_samp2,
+      GD_INT_TYPE, cntbuf);
+
+  if (D->error != GD_E_OK) {
+    free(cntbuf);
+    free(tmpbuf);
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* enmultiplex the data */
+  _GD_MplexOutData(D, tmpbuf, spf1, cntbuf, spf2, data_in, data_type,
+      E->EN(mplex,count_val), num_samp);
+  free(cntbuf);
+
+  /* and write it. */
+  if (!D->error)
+    n_wrote = _GD_DoFieldOut(D, E->e->entry[0], first_samp, num_samp, data_type,
+        tmpbuf);
+  free(tmpbuf);
+
+  dreturn("%" PRNsize_t, n_wrote);
+  return n_wrote;
+}
+
+static size_t _GD_DoConstOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first, size_t len, gd_type_t data_type,
+    const void *restrict data_in)
+{
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first, len,
+      data_type, data_in);
 
   /* check protection */
   if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT)
@@ -543,64 +679,38 @@ static size_t _GD_DoConstOut(DIRFILE* D, gd_entry_t *E, off64_t first,
   return 1;
 }
 
-static size_t _GD_DoStringOut(DIRFILE* D, gd_entry_t *E, const char *data_in)
-{
-  char* ptr = E->e->u.string;
-
-  dtrace("%p, %p, %p", D, E, data_in);
-
-  /* check protection */
-  if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT) {
-    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
-        D->fragment[E->fragment_index].cname);
-    dreturn("%i", 0);
-    return 0;
-  }
-
-  E->e->u.string = strdup(data_in);
-  if (E->e->u.string == NULL) {
-    E->e->u.string = ptr;
-    _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
-    dreturn("%i", 0);
-    return 0;
-  }
-  free(ptr);
-  D->fragment[E->fragment_index].modified = 1;
-
-  dreturn("%zu", strlen(E->e->u.string) + 1);
-  return strlen(E->e->u.string) + 1;
-}
-
-size_t _GD_DoFieldOut(DIRFILE *D, gd_entry_t* E, int repr, off64_t first_samp,
-    size_t num_samp, gd_type_t data_type, const void *data_in)
+size_t _GD_DoFieldOut(DIRFILE *restrict D, gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t data_type,
+    const void *restrict data_in)
 {
   size_t n_wrote = 0;
 
-  dtrace("%p, %p, %i, %lli, %zu, 0x%x, %p", D, E, repr, first_samp, num_samp,
-      data_type, data_in);
+  dtrace("%p, %p, %lli, %" PRNsize_t ", 0x%X, %p", D, E, (long long)first_samp,
+      num_samp, data_type, data_in);
 
   if (++D->recurse_level >= GD_MAX_RECURSE_LEVEL) {
-    _GD_SetError(D, GD_E_RECURSE_LEVEL, 0, NULL, 0, E->field);
+    _GD_SetError(D, GD_E_RECURSE_LEVEL, GD_E_RECURSE_CODE, NULL, 0, E->field);
     D->recurse_level--;
     dreturn("%i", 0);
     return 0;
   }
 
-  if (!E->e->calculated)
-    _GD_CalculateEntry(D, E);
+  if (!(E->flags & GD_EN_CALC)) {
+    _GD_CalculateEntry(D, E, 1);
 
-  if (D->error) {
-    dreturn("%i", 0);
-    return 0;
+    if (D->error) {
+      D->recurse_level--;
+      dreturn("%i", 0);
+      return 0;
+    }
   }
 
-  /* writing to representations is prohibited */
-  if (repr != GD_REPR_NONE) {
-    const char r[2] = {(char)repr, 0};
-    _GD_SetError(D, GD_E_BAD_REPR, GD_E_REPR_PUT, NULL, 0, r);
-    dreturn("%i", 0);
-    return 0;
-  }
+  /* this call will throw GD_E_DOMAIN if a problem arises; however, that only
+   * happens in cases where the field has multiple inputs, which putdata will
+   * reject anyways; so we ignore this error for a more relevant one later
+   */
+  if (first_samp == GD_HERE)
+    first_samp = _GD_GetFilePos(D, E, -1);
 
   switch (E->field_type) {
     case GD_RAW_ENTRY:
@@ -617,10 +727,8 @@ size_t _GD_DoFieldOut(DIRFILE *D, gd_entry_t* E, int repr, off64_t first_samp,
     case GD_SBIT_ENTRY:
       n_wrote = _GD_DoBitOut(D, E, first_samp, num_samp, data_type, data_in);
       break;
-    case GD_MULTIPLY_ENTRY:
-    case GD_DIVIDE_ENTRY:
-    case GD_INDEX_ENTRY:
-      _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_PUT, NULL, 0, E->field);
+    case GD_MPLEX_ENTRY:
+      n_wrote = _GD_DoMplexOut(D, E, first_samp, num_samp, data_type, data_in);
       break;
     case GD_RECIP_ENTRY:
       n_wrote = _GD_DoRecipOut(D, E, first_samp, num_samp, data_type, data_in);
@@ -636,32 +744,36 @@ size_t _GD_DoFieldOut(DIRFILE *D, gd_entry_t* E, int repr, off64_t first_samp,
     case GD_CARRAY_ENTRY:
       n_wrote = _GD_DoConstOut(D, E, first_samp, num_samp, data_type, data_in);
       break;
-    case GD_STRING_ENTRY:
-      n_wrote = _GD_DoStringOut(D, E, (const char *)data_in);
+    case GD_MULTIPLY_ENTRY:
+    case GD_DIVIDE_ENTRY:
+    case GD_WINDOW_ENTRY:
+    case GD_INDEX_ENTRY:
+      _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_PUT, NULL, 0, E->field);
       break;
+    case GD_STRING_ENTRY:
+    case GD_ALIAS_ENTRY:
     case GD_NO_ENTRY:
       _GD_InternalError(D);
       break;
   }
 
   D->recurse_level--;
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
   return n_wrote;
 }
 
 /* this function is little more than a public boilerplate for _GD_DoFieldOut */
-size_t gd_putdata64(DIRFILE* D, const char *field_code_in, off64_t first_frame,
+size_t gd_putdata64(DIRFILE* D, const char *field_code, off64_t first_frame,
     off64_t first_samp, size_t num_frames, size_t num_samp, gd_type_t data_type,
     const void *data_in)
 {
   size_t n_wrote = 0;
   gd_entry_t *entry;
-  char* field_code;
-  int repr;
-  gd_spf_t spf;
+  unsigned int spf = 0;
 
-  dtrace("%p, \"%s\", %lli, %lli, %zu, %zu, 0x%x, %p", D, field_code_in,
-      first_frame, first_samp, num_frames, num_samp, data_type, data_in);
+  dtrace("%p, \"%s\", %lli, %lli, %" PRNsize_t ", %" PRNsize_t ", 0x%X, %p", D,
+      field_code, (long long)first_frame, (long long)first_samp, num_frames,
+      num_samp, data_type, data_in);
 
   if (D->flags & GD_INVALID) {/* don't crash */
     _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
@@ -677,39 +789,55 @@ size_t gd_putdata64(DIRFILE* D, const char *field_code_in, off64_t first_frame,
 
   _GD_ClearError(D);
 
-  entry = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1);
+  entry = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
 
-  if (D->error) {
+  if (entry == NULL) {
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
     dreturn("%i", 0);
     return 0;
   }
 
-  if (entry->field_type & GD_SCALAR_ENTRY)
+  if (entry->field_type & GD_SCALAR_ENTRY_BIT)
     _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_CALLER, NULL, 0, field_code);
 
-  if (field_code != field_code_in)
-    free(field_code);
-
   if (D->error) {
     dreturn("%i", 0);
     return 0;
   }
 
-  /* get the samples per frame */
-  spf = _GD_GetSPF(D, entry);
+  if (first_frame == GD_HERE || first_samp == GD_HERE) {
+    first_samp = GD_HERE;
+    first_frame = 0;
+  }
 
-  if (D->error) {
+  if (num_frames > 0 || first_frame > 0) {
+    /* get the samples per frame */
+    spf = _GD_GetSPF(D, entry);
+
+    if (D->error) {
+      dreturn("%i", 0);
+      return 0;
+    }
+
+    first_samp += spf * first_frame;
+    num_samp += spf * num_frames;
+  }
+
+  if (first_samp < 0 && (first_samp != GD_HERE || first_frame != 0)) {
+    _GD_SetError(D, GD_E_RANGE, GD_E_OUT_OF_RANGE, NULL, 0, NULL);
     dreturn("%i", 0);
     return 0;
   }
 
-  first_samp += spf * first_frame;
-  num_samp += spf * num_frames;
+  /* the easy case */
+  if (num_samp == 0) {
+    dreturn("%u", 0);
+    return 0;
+  }
+ 
+  n_wrote = _GD_DoFieldOut(D, entry, first_samp, num_samp, data_type, data_in);
 
-  n_wrote = _GD_DoFieldOut(D, entry, repr, first_samp, num_samp, data_type,
-      data_in);
-
-  dreturn("%zu", n_wrote);
+  dreturn("%" PRNsize_t, n_wrote);
   return n_wrote;
 }
 
