@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2010 D. V. Wiebe
+ * Copyright (C) 2005-2010, 2012, 2014 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -21,15 +21,6 @@
  */
 #include "internal.h"
 
-#ifdef STDC_HEADERS
-#include <inttypes.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-#endif
-
 #ifdef GD_NO_C99_API
 
 #define TO_COMPLEX(ot,it) \
@@ -66,13 +57,14 @@
 
 /* _GD_ConvertType: copy data to output buffer while converting type.
 */
-void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
-    void *data_out, gd_type_t out_type, size_t n) gd_nothrow
+void _GD_ConvertType(DIRFILE *restrict D, const void *restrict data_in,
+    gd_type_t in_type, void *restrict data_out, gd_type_t out_type, size_t n)
+gd_nothrow
 {
   size_t i;
 
-  dtrace("%p, %p, 0x%x, %p, 0x%x, %zu", D, data_in, in_type, data_out, out_type,
-      n);
+  dtrace("%p, %p, 0x%x, %p, 0x%x, %" PRNsize_t, D, data_in, in_type, data_out,
+      out_type, n);
   dreturnvoid();
 
   if (out_type == GD_NULL) /* null return type: don't return data */
@@ -641,6 +633,7 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
           break;
       }
+      break;
     case GD_COMPLEX128:
       switch (out_type) {
         case GD_INT8:
