@@ -1,3 +1,23 @@
+/* Copyright (C) 2010-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Test field modifying */
 #include "test.h"
 
@@ -12,19 +32,20 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/data";
-  const char* format_data = "data RAW INT32 8\nphase PHASE data 1\n"
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/data";
+  const char *format_data = "data RAW INT32 8\nphase PHASE data 1\n"
     "lincom LINCOM 3 data 1 3 phase 2 0 data 3 1\n";
   int32_t data_data[256];
   int32_t c[8];
   int fd, i, ret, error, n, r = 0;
   DIRFILE *D;
-  const char* in_fields[2] = {"data", "phase"};
+  const char *in_fields[2] = {"data", "phase"};
   const double m[2] = {2, 4};
   const double b[2] = {0, 2};
 
+  rmdirfile();
   mkdir(filedir, 0777);
 
   for (fd = 0; fd < 256; ++fd)
@@ -43,7 +64,7 @@ int main(void)
   error = gd_error(D);
   n = gd_getdata(D, "lincom", 5, 0, 1, 0, GD_INT32, c);
 
-  gd_close(D);
+  gd_discard(D);
 
   for (i = 0; i < 8; ++i)
     CHECKIi(i,c[i], (40 + i) * 6 + 6);
