@@ -1,3 +1,23 @@
+/* Copyright (C) 2010-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 #include "test.h"
 
 #include <stdlib.h>
@@ -9,10 +29,10 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/data";
-  const char* format_data =
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/data";
+  const char *format_data =
     "data RAW UINT16 1\n"
     "mult1 MULTIPLY data INDEX\n"
     "mult2 MULTIPLY INDEX INDEX\n"
@@ -22,6 +42,7 @@ int main(void)
   off_t eof_INDEX, eof_mult1, eof_mult2, eof_mult3;
   DIRFILE *D;
 
+  rmdirfile();
   mkdir(filedir, 0777);
 
   fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
@@ -41,7 +62,7 @@ int main(void)
   error2 = gd_error(D);
   eof_mult3 = gd_eof(D, "mult3");
   error3 = gd_error(D);
-  gd_close(D);
+  gd_discard(D);
 
   unlink(data);
   unlink(format);
