@@ -1,3 +1,23 @@
+/* Copyright (C) 2008-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Retreiving the number of fields of a field should succeed cleanly */
 #include "test.h"
 
@@ -11,9 +31,9 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* format_data =
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *format_data =
     "data1 STRING UINT8 1\n"
     "data2 STRING UINT8 1\n"
     "data3 STRING UINT8 1\n"
@@ -22,6 +42,7 @@ int main(void)
   const char **field_list;
   DIRFILE *D;
 
+  rmdirfile();
   mkdir(filedir, 0777);
 
   fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
@@ -54,7 +75,7 @@ int main(void)
 
   CHECKI(i, 3);
 
-  gd_close(D);
+  gd_discard(D);
   unlink(format);
   rmdir(filedir);
 
