@@ -1,3 +1,23 @@
+/* Copyright (C) 2008-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Attempt to read UINT64 with the opposite endianness */
 #include "test.h"
 
@@ -22,9 +42,9 @@ static int BigEndian(void)
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/data";
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/data";
   char format_data[1000];
   uint64_t c = 0;
   uint64_t data_data[128];
@@ -32,13 +52,14 @@ int main(void)
   int fd, n, error, r = 0;
   DIRFILE *D;
 
+  rmdirfile();
   mkdir(filedir, 0777); 
 
   sprintf(format_data, "data RAW UINT64 1\nENDIAN %s\n", (big_endian)
       ? "little" : "big");
 
   for (fd = 0; fd < 128; ++fd)
-    data_data[fd] = fd * (0x020100000201LLU);
+    data_data[fd] = fd * (0x020100000201);
 
   fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
   write(fd, format_data, strlen(format_data));
@@ -52,7 +73,7 @@ int main(void)
   n = gd_getdata(D, "data", 5, 0, 1, 0, GD_UINT64, &c);
   error = gd_error(D);
 
-  gd_close(D);
+  gd_discard(D);
 
   unlink(data);
   unlink(format);
@@ -60,7 +81,7 @@ int main(void)
 
   CHECKI(error, 0);
   CHECKI(n, 1);
-  CHECKX(c, 0x50a0000050a0000LLU);
+  CHECKX(c, 0x50a0000050a0000);
 
   return r;
 }
