@@ -1,5 +1,24 @@
+/* Copyright (C) 2009-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Retreiving the number of frames should succeed cleanly */
-#include "../src/config.h"
 #include "test.h"
 
 #include <stdlib.h>
@@ -15,17 +34,18 @@ int main(void)
 #ifndef TEST_LZMA
   return 77;
 #else
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/data";
-  const char* xzdata = __TEST__ "dirfile/data.lzma";
-  const char* format_data = "data RAW UINT16 1\n";
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/data";
+  const char *xzdata = "dirfile/data.lzma";
+  const char *format_data = "data RAW UINT16 1\n";
   char command[4096];
   uint16_t data_data[256];
   int i, error, r = 0;
   DIRFILE *D;
   size_t n;
 
+  rmdirfile();
   mkdir(filedir, 0777);
 
   for (i = 0; i < 256; ++i)
@@ -51,7 +71,7 @@ int main(void)
 #endif
   n = gd_nframes(D);
   error = gd_error(D);
-  gd_close(D);
+  gd_discard(D);
 
   unlink(xzdata);
   unlink(format);
