@@ -1,3 +1,23 @@
+/* Copyright (C) 2009-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Attempt to write little-endian COMPLEX64 */
 #include "test.h"
 
@@ -12,10 +32,10 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/data";
-  const char* format_data = "data RAW COMPLEX64 1\nENDIAN little\n";
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/data";
+  const char *format_data = "data RAW COMPLEX64 1\nENDIAN little\n";
   unsigned int i;
 #ifdef GD_NO_C99_API
   const float c[] = {1.5, 2.25};
@@ -29,6 +49,7 @@ int main(void)
   int fd, n, error, r = 0;
   DIRFILE *D;
 
+  rmdirfile();
   mkdir(filedir, 0777); 
 
   fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
@@ -43,7 +64,7 @@ int main(void)
 #endif
   error = gd_error(D);
 
-  gd_close(D);
+  gd_discard(D);
 
   fd = open(data, O_RDONLY);
   lseek(fd, 5 * 2 * sizeof(float), SEEK_SET);
