@@ -1,3 +1,23 @@
+/* Copyright (C) 2009-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Attempt to read COMPLEX128 as FLOAT64 */
 #include "test.h"
 
@@ -12,10 +32,10 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/data";
-  const char* format_data = "data RAW COMPLEX128 8\n";
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/data";
+  const char *format_data = "data RAW COMPLEX128 8\n";
 #ifdef GD_NO_C99_API
   double data_data[256][2];
 #else
@@ -26,6 +46,7 @@ int main(void)
   DIRFILE *D;
 
   memset(c, 0, 8);
+  rmdirfile();
   mkdir(filedir, 0777);
 
   for (fd = 0; fd < 256; ++fd) {
@@ -48,7 +69,7 @@ int main(void)
   n = gd_getdata(D, "data", 5, 0, 1, 0, GD_FLOAT64, c);
   error = gd_error(D);
 
-  gd_close(D);
+  gd_discard(D);
 
   unlink(data);
   unlink(format);
