@@ -1,3 +1,23 @@
+/* Copyright (C) 2010-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Open a Standards Version 7 conformant dirfile */
 #include "test.h"
 
@@ -11,16 +31,17 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/ar";
-  const char* format_data = "/VERSION 7\nar RAW UINT8 8\nar/q SBIT ar 0 10\n";
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/ar";
+  const char *format_data = "/VERSION 7\nar RAW UINT8 8\nar/q SBIT ar 0 10\n";
   uint16_t c[8];
   unsigned char data_data[256];
   int fd, i, n, error, v, l, e, r = 0;
   DIRFILE *D;
 
   memset(c, 0, 8);
+  rmdirfile();
   mkdir(filedir, 0777);
 
   for (fd = 0; fd < 256; ++fd)
@@ -42,7 +63,7 @@ int main(void)
   l = gd_dirfile_standards(D, GD_VERSION_LATEST);
   e = gd_dirfile_standards(D, GD_VERSION_EARLIEST);
 
-  gd_close(D);
+  gd_discard(D);
 
   unlink(data);
   unlink(format);
@@ -56,7 +77,7 @@ int main(void)
 
   /* Version 7 is forward compatible with version 8 */
   CHECKI(v,7);
-  CHECKI(l,8);
+  CHECKI(l,GD_DIRFILE_STANDARDS_VERSION);
   CHECKI(e,7);
 
   return r;
