@@ -1,3 +1,23 @@
+/* Copyright (C) 2010-2011, 2013 D. V. Wiebe
+ *
+ ***************************************************************************
+ *
+ * This file is part of the GetData project.
+ *
+ * GetData is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License as published by the
+ * Free Software Foundation; either version 2.1 of the License, or (at your
+ * option) any later version.
+ *
+ * GetData is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with GetData; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 /* Open a Standards Version 8 conformant dirfile */
 #include "test.h"
 
@@ -11,10 +31,10 @@
 
 int main(void)
 {
-  const char* filedir = __TEST__ "dirfile";
-  const char* format = __TEST__ "dirfile/format";
-  const char* data = __TEST__ "dirfile/ar";
-  const char* format_data =
+  const char *filedir = "dirfile";
+  const char *format = "dirfile/format";
+  const char *data = "dirfile/ar";
+  const char *format_data =
     "/VERSION 8\n"
     "ar RAW UINT8 8\n"
     "FRAMEOFFSET DIVIDE ar ar\n"
@@ -25,6 +45,7 @@ int main(void)
   DIRFILE *D;
 
   memset(c, 0, 8);
+  rmdirfile();
   mkdir(filedir, 0777);
 
   for (fd = 0; fd < 256; ++fd)
@@ -46,7 +67,7 @@ int main(void)
   l = gd_dirfile_standards(D, GD_VERSION_LATEST);
   e = gd_dirfile_standards(D, GD_VERSION_EARLIEST);
 
-  gd_close(D);
+  gd_discard(D);
 
   unlink(data);
   unlink(format);
@@ -59,7 +80,7 @@ int main(void)
     CHECKFi(i,c[i],1. / (40 + i));
 
   CHECKI(v,8);
-  CHECKI(l,8);
+  CHECKI(l,GD_DIRFILE_STANDARDS_VERSION);
   CHECKI(e,8);
 
   return r;
