@@ -1,5 +1,5 @@
 /* Copyright (C) 2010, 2011 Matthew Truch
- * Copyright (C) 2010 D. V. Wiebe
+ * Copyright (C) 2010, 2012, 2014 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -17,23 +17,9 @@
  * along with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "internal.h"
 
-#define _XOPEN_SOURCE 1000
-#define __STDC_FORMAT_MACROS
-#define __STDC_LIMIT_MACROS
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdint.h>
-#include <inttypes.h>
 #include <getopt.h>
-#include <math.h>
-#include "nan.h"
-#include "getdata.h"
 
 #ifndef HAVE_OFF64_T
 #define off64_t off_t
@@ -47,7 +33,7 @@ struct field
 {
   char *name;
   int type;
-  gd_spf_t spf;
+  unsigned int spf;
   union {
     double *dbl;
     int64_t *i64;
@@ -60,7 +46,7 @@ struct field
 void version(void)
 {
   printf("dirfile2ascii (%s)\n"
-      "Copyright (C) 2010 Matthew Truch\n\n"
+      "Copyright (C) 2010, 2011, 2012, 2014 Matthew Truch and others\n\n"
       "Please send reports of bugs and other communication to:\n  %s\n\n"
       "This program comes with NO WARRANTY, not even for MERCHANTABILITY "
       "or FITNESS\n"
@@ -159,9 +145,9 @@ int main (int argc, char **argv)
   size_t nf = 0;
   off64_t ff = 0;
   size_t n_want;
-  gd_spf_t j;
-  gd_spf_t max_spf = 0;
-  gd_spf_t min_spf = UINT16_MAX;
+  unsigned int j;
+  unsigned int max_spf = 0;
+  unsigned int min_spf = INT_MAX;
   int c, i;
   int numfields = 0;
   struct field fields[BUF_LEN];
@@ -235,7 +221,7 @@ int main (int argc, char **argv)
         if (dirfile_name == NULL) {
           dirfile_name = optarg;
         } else { /* Standard output field */
-          if (numfields > BUF_LEN) {
+          if (numfields >= BUF_LEN) {
             fprintf(stderr, "Error: Too many fields!\n");
             exit(-2);
           }
@@ -259,7 +245,7 @@ int main (int argc, char **argv)
       case 'F':
       case 'A':
       case 'a':
-        if (numfields > BUF_LEN) {
+        if (numfields >= BUF_LEN) {
           fprintf(stderr, "Error: Too many fields!\n");
           exit(-2);
         }
@@ -481,10 +467,10 @@ int main (int argc, char **argv)
           /*                 val= y0 +  diff  *     slope       */
           /* Note that if max_spfs isn't a multiple of all spfs then some of  */
           /* the data will only be approximated (except at frame boundaries). */
-          gd_spf_t prev_samp, next_samp, offset;
+          unsigned int prev_samp, next_samp, offset;
           double slope, diff, val;
 
-          prev_samp = (gd_spf_t)floor((double)j * (double)fields[i].spf /
+          prev_samp = (unsigned int)floor((double)j * (double)fields[i].spf /
               (double)max_spf);
           next_samp = prev_samp + 1;
           diff = ((double)prev_samp + (double)(j % (max_spf / fields[i].spf)) /
